App.Medicine.OrganFarm.Organ = class {
	/**
	 * @param {Object} params
	 * @param {string} params.type - unique type of organ, used as key
	 * @param {string} params.name - display name
	 * @param {string|function(App.Entity.SlaveState):string} [params.tooltip] - full sentence, uncapitalized and unpunctuated
	 * @param {number} params.cost - how much it costs to grow the organ
	 * @param {number} params.time - how long it takes to grow the organ (without upgrades)
	 * @param {function(App.Entity.SlaveState):boolean} [params.canGrow]
	 * @param {string[]} [params.dependencies] - organs that are implanted first if possible, use type of other organs as values
	 * @param {App.Medicine.OrganFarm.OrganImplantAction[]} [params.actions]
	 * @param {boolean} [params.displayMultipleActions=false] allow multiple implant links to be displayed
	 */
	constructor({
		type, name, tooltip = "", cost, time, canGrow = () => true, dependencies = [],
		displayMultipleActions = false, actions = []
	}) {
		this.type = type;
		this.name = name;
		this.tooltip = tooltip;
		this.cost = cost;
		this.time = time;
		/** @type {function(App.Entity.SlaveState):boolean} */
		this.canGrow = canGrow;
		/** @type {string[]} */
		this.dependencies = dependencies;
		this.displayMultipleActions = displayMultipleActions;
		/** @type {App.Medicine.OrganFarm.OrganImplantAction[]} */
		this.implantActions = actions;

		App.Medicine.OrganFarm.Organs[type] = this;
	}
};

App.Medicine.OrganFarm.OrganImplantAction = class {
	/**
	 * @param {object} params
	 * @param {string} params.name - display name
	 * @param {string} [params.tooltip] - full sentence, uncapitalized and unpunctuated
	 * @param {number} params.healthImpact
	 * @param {string} params.surgeryType - for use with surgery degradation
	 * @param {boolean} [params.autoImplant]
	 * @param {function(App.Entity.SlaveState):boolean} params.canImplant
	 * @param {function(App.Entity.SlaveState):string} params.implantError - message to show if this action cannot be used
	 * @param {function(App.Entity.SlaveState):void} params.implant
	 */
	constructor({
		name, tooltip = "", healthImpact, surgeryType, autoImplant = true, canImplant, implantError, implant
	}) {
		this.name = name;
		this.tooltip = tooltip;
		this.healthImpact = healthImpact;
		this.surgeryType = surgeryType;
		this.autoImplant = autoImplant;
		/**
		 * True if this action can implant the organ
		 * @type {function(App.Entity.SlaveState):boolean}
		 */
		this.canImplant = canImplant;
		/**
		 * Error message if the organ cannot be implanted.
		 * @type {function(App.Entity.SlaveState):string}
		 */
		this.implantError = implantError;
		/**
		 * Implant the organ
		 * @type {function(App.Entity.SlaveState):void}
		 */
		this.implant = implant;
	}
};
