/**
 * @returns {DocumentFragment}
 */
App.EndWeek.arcadeReport = function() {
	const el = new DocumentFragment();
	let r;

	const arcadeStats = document.createElement("span");
	el.append(arcadeStats);

	const slaves = App.Utils.sortedEmployees(App.Entity.facilities.arcade);
	const slavesLength = slaves.length;
	let cockmilked = 0, milked = 0, milkProfits = 0, profits = 0, boobsImplanted = 0, prostatesImplanted = 0, vasectomiesUndone = 0, mSlave, bSlave, pSlave, cSlave, milkResults, growth;

	// Statistics gathering
	V.facility = V.facility || {};
	V.facility.arcade = initFacilityStatistics(V.facility.arcade);
	const arcadeNameCaps = capFirstChar(V.arcadeName);

	if (slavesLength > 1) {
		App.UI.DOM.appendNewElement("p", el, `There are ${slavesLength} inmates confined in ${V.arcadeName}.`, ["bold", "indent"]);
	} else {
		App.UI.DOM.appendNewElement("p", el, `There is one inmate confined in ${V.arcadeName}.`, ["bold", "indent"]);
	}
	r = [];
	if (V.arcologies[0].FSDegradationist > 20) {
		if (V.arcologies[0].FSDegradationistLaw === 1) {
			r.push(`The tenants located near the arcade consider the facility a normal part of their surroundings, including the crowd of menial slaves waiting their turn spills out into the hallway, 24 hours a day.`);
		} else {
			r.push(`The tenants located near the arcade consider the facility a normal part of their surroundings.`);
		}
	} else if (V.arcologies[0].FSPaternalist > 20) {
		r.push(`The tenants located near the arcade consider it an <span class="red">intolerable</span> establishment.`);
	}
	App.Events.addNode(el, r, "div", "indent");
	if (V.arcadeDecoration !== "standard") {
		App.Events.addNode(el, [`${arcadeNameCaps}'s customers enjoy <span class="green">getting off in ${V.arcadeDecoration} surroundings.</span>`], "p", "indent");
	}

	for (const slave of slaves) {
		const {He} = getPronouns(slave);
		r = [];
		/* for the included passages */

		/* Perform facility based rule changes */
		slave.clothes = "no clothing";
		/* Health */
		if (V.arcadeUpgradeHealth === 2 && slave.health.condition < 40) {
			improveCondition(slave, 2);
		} else if (slave.health.condition > 20) {
			healthDamage(slave, 5);
		} else if (V.arcadeUpgradeHealth === 1 && slave.health.condition < -30) {
			improveCondition(slave, 2);
		} else if (slave.health.condition > -50) {
			healthDamage(slave, 3);
		} else if (slave.health.condition > -90) {
			healthDamage(slave, 1);
		}
		/* Curatives */
		if (V.arcadeUpgradeHealth === 2) {
			slave.curatives = 2;
		} else if (V.arcadeUpgradeHealth === 1) {
			slave.curatives = 1;
		}
		/* Stimulation or lack thereof trust/devotion effect */
		if (V.arcadeUpgradeInjectors > 0) {
			if (V.arcadeUpgradeInjectors === 2) {
				slave.aphrodisiacs = 2;
			}
			slave.devotion -= 5;
			slave.trust -= 10;
		} else {
			slave.trust -= 5;
		}
		/* Muscles */
		if (slave.muscles > -100) {
			slave.muscles--;
		}
		const oldCash = V.cash;
		if (V.showEWD !== 0) {
			const {He} = getPronouns(slave);
			const slaveEntry = App.UI.DOM.appendNewElement("div", el, '', "slave-report");
			App.SlaveAssignment.appendSlaveArt(slaveEntry, slave);
			slaveEntry.append(App.UI.favoriteToggle(slave), " ");
			r = [];
			r.push(App.UI.DOM.makeElement("span", SlaveFullName(slave), "slave-name"));
			if (slave.choosesOwnAssignment === 2) {
				r.push(App.SlaveAssignment.choosesOwnJob(slave));
			} else {
				r.push(`is confined in ${V.arcadeName}.`);
			}
			App.Events.addNode(slaveEntry, r);

			App.Events.addNode(
				slaveEntry,
				[
					He,
					App.SlaveAssignment.workAGloryHole(slave),
				],
				"div",
				"indent"
			);
		} else {
			// discard return values silently
			App.SlaveAssignment.choosesOwnJob(slave);
			App.SlaveAssignment.workAGloryHole(slave);
		}
		profits += V.cash - oldCash;

		if (V.arcadeUpgradeCollectors > 0) {
			if (slave.vasectomy === 1) {
				slave.vasectomy = 0;
				vasectomiesUndone++;
				cashX(forceNeg(V.surgeryCost), "slaveSurgery", slave);
				surgeryDamage(slave, 10);
			} else if (slave.lactation < 2) {
				slave.lactation = 2;
				boobsImplanted++;
				bSlave = slave;
				cashX(forceNeg(V.surgeryCost), "slaveSurgery", slave);
				surgeryDamage(slave, 10);
			} else if (slave.prostate === 1) {
				slave.prostate = 2;
				prostatesImplanted++;
				pSlave = slave;
				cashX(forceNeg(V.surgeryCost), "slaveSurgery", slave);
				surgeryDamage(slave, 10);
			} else if (slave.lactation > 0 || slave.balls > 0) {
				milkResults = App.SlaveAssignment.getMilked(slave, 1.0);
				if (V.showEWD !== 0) {
					r = [];
					r.push(App.UI.DOM.makeElement("div", `${He} ${milkResults.text}`, "indent"));
					App.Events.addNode(el, r, "div", "indent");
				}
				milkProfits += milkResults.cash;
				growth = 0;
				const gigantomastiaMod = slave.geneticQuirks.gigantomastia === 2 ? (slave.geneticQuirks.macromastia === 2 ? 3 : 2) : 1;
				if (slave.boobs < 2000) {
					growth = 100;
				} else if (slave.boobs < 5000 * gigantomastiaMod) {
					growth = 50;
				} else if (slave.boobs < 10000 * gigantomastiaMod) {
					growth = 25;
				}
				if (slave.geneMods.NCS === 1) {
					/*
					** NCS will allow some growth for Arcade milking, but not as much as the Dairy.
					*/
					growth = Math.trunc(growth / 3.5);
				}
				slave.boobs += growth;
				if (
					(slave.balls > 0) &&
					(slave.balls < 10) &&
					(random(1, 100) > (40 + (10 * (slave.balls + (2 * slave.geneMods.NCS)))))
				) {
					slave.balls++;
				}
				if (
					(slave.dick > 0) &&
					(slave.dick < 10) &&
					(random(1, 100) > (40 + (10 * slave.dick + (2 * slave.geneMods.NCS))))
				) {
					slave.dick++;
				}
				if (slave.lactation > 0) {
					milked++;
					mSlave = slave;
				}
				if (slave.balls > 0) {
					cockmilked++;
					cSlave = slave;
				}
			}
		}
		if (slave.inflation > 0) {
			deflate(slave);
		}
		if (V.showEWD !== 0) {
			el.append(App.SlaveAssignment.standardSlaveReport(slave, false));
		} else {
			App.SlaveAssignment.standardSlaveReport(slave, true);
		}
	}

	if (slavesLength + V.fuckdolls > 0) {
		r = [];
		if (milked === 1) {
			const {his} = getPronouns(mSlave);
			r.push(`One of them is lactating and spends ${his} time in ${V.arcadeName} being simultaneously milked and fucked.`);
		} else if (milked > 1) {
			r.push(`${capFirstChar(num(milked))} of them are lactating and spend their time in ${V.arcadeName} being simultaneously milked and fucked.`);
		}

		if (vasectomiesUndone) {
			if (vasectomiesUndone === 1) {
				r.push(`One`);
			} else {
				r.push(capFirstChar(num(vasectomiesUndone)));
			}
			r.push(`of them had severed vas deferens, so they were reattached to allow sperm through, costing <span class="red">${cashFormat(V.surgeryCost * vasectomiesUndone)}.</span>`);
		}
		if (boobsImplanted) {
			if (boobsImplanted === 1) {
				const {he} = getPronouns(bSlave);
				r.push(`One of them was not lactating, so ${he} was`);
			} else {
				r.push(`${capFirstChar(num(boobsImplanted))} of them were not lactating, so they were`);
			}
			r.push(`implanted with long-acting lactation inducing drugs, costing <span class="red">${cashFormat(V.surgeryCost * boobsImplanted)}.</span>`);
		}
		if (prostatesImplanted) {
			if (prostatesImplanted === 1) {
				const {he} = getPronouns(pSlave);
				r.push(`One of them was not producing the maximum possible amount of precum, so ${he} was`);
			} else {
				r.push(`${capFirstChar(num(prostatesImplanted))} of them were not producing the maximum possible amount of precum, so they were`);
			}
			r.push(`implanted with long-acting prostate stimulation drugs, costing <span class="red">${cashFormat(V.surgeryCost * prostatesImplanted)}.</span>`);
		}
		if (cockmilked === 1) {
			const {he} = getPronouns(cSlave);
			r.push(`One of them retains testicles and is brutally cockmilked as ${he} is used.`);
		} else if (cockmilked > 1) {
			r.push(`${capFirstChar(num(cockmilked))} of them retain testicles and are brutally cockmilked as they are used.`);
		}

		r.push(`The arcade makes you`);
		if (V.policies.publicFuckdolls === 0) {
			r.push(`<span class="yellowgreen">${cashFormat(profits)}</span> from selling the inmates' holes`);
		} else {
			r.push(`<span class="green">more reputable</span> from freely providing the inmates' holes`);
		}
		if (V.arcadeUpgradeCollectors > 0 && V.policies.publicFuckdolls === 0) {
			r.push(`and`);
		}
		if (V.arcadeUpgradeCollectors > 0) {
			r.push(`<span class="yellowgreen">${cashFormat(milkProfits)}</span> from selling the fluids they produced`);
		}
		r.push(`this week.`);
		if (V.arcologies[0].FSPaternalist > 20) {
			repX(forceNeg(Math.trunc(profits / 20)), "arcade");
		}
		App.Events.addNode(el, r, "div", "indent");
	}

	if (V.arcadeUpgradeFuckdolls > 1) {
		const fuckdolls = slaves.filter(s => fuckdollQualifier(s));
		if (fuckdolls.length > 0) {
			// ascending sort by sexAmount...least popular slaves are converted first
			fuckdolls.sort((a, b) => b.sexAmount - a.sexAmount);
			for (const fuckdoll of fuckdolls) {
				const {he} = getPronouns(fuckdoll);
				App.UI.DOM.appendNewElement("div", el, `${fuckdoll.slaveName} is low-quality merchandise, so ${he} has been converted into a Fuckdoll.`, "indent");
				removeSlave(fuckdoll);
				V.fuckdolls++;
				if (V.arcadeUpgradeFuckdolls === 2) {
					break; // convert at most one per week on this setting
				}
			}
		} else {
			App.UI.DOM.appendNewElement("div", el, `No slaves have failed quality inspection for Fuckdoll conversion. ${arcadeNameCaps} will remain overcrowded this week.`, "indent");
		}
	}

	if (slavesLength > 0) {
		// Record statistics gathering
		let b = V.facility.arcade;
		b.whoreIncome = 0;
		b.customers = 0;
		b.whoreCosts = 0;
		b.rep = 0;
		for (let si of b.income.values()) {
			b.whoreIncome += si.income;
			b.customers += si.customers;
			b.whoreCosts += si.cost;
			b.rep += si.rep;
		}
		b.maintenance = V.arcade * V.facilityCost * (0.05 + 0.02 * V.arcadeUpgradeInjectors + 0.05 * V.arcadeUpgradeCollectors);
		b.totalIncome = b.whoreIncome;
		b.totalExpenses = b.whoreCosts + b.maintenance;
		b.profit = b.totalIncome - b.totalExpenses;


		// Arcade stats
		el.append(App.Facilities.Arcade.Stats(false));
		arcadeStats.append(App.Facilities.Arcade.Stats(true));
	}
	return el;

	/**
	 *
	 * @param {App.Entity.SlaveState} slave
	 */
	function fuckdollQualifier(slave) {
		if (slave.sentence === 0 && slave.fuckdoll === 0 && slave.fetish === "mindbroken") {
			if (slave.physicalAge > 35) {
				return true;
			} else if (slave.vagina >= 4 || slave.anus >= 4) {
				return true;
			}
		}
	}
};
