/**
 * Creates a full report of the Nursery
 * @returns {DocumentFragment}
 */
App.Facilities.Nursery.nurseryReport = function nurseryReport() {
	const frag = new DocumentFragment();

	const slaves = App.Utils.sortedEmployees(App.Entity.facilities.nursery);
	const devBonus = (V.nurseryDecoration !== "standard") ? 1 : 0;
	const arcology = V.arcologies[0];

	let matronBonus = 0;

	V.flSex = App.EndWeek.getFLSex(App.Entity.facilities.nursery); // FIXME: should be local, passed as a parameter to saRules

	function matronChanges() {
		if (S.Matron) {
			S.Matron.devotion += devBonus;

			if (S.Matron.health.condition < -80) {
				improveCondition(S.Matron, 20);
			} else if (S.Matron.health.condition < -40) {
				improveCondition(S.Matron, 15);
			} else if (S.Matron.health.condition < 0) {
				improveCondition(S.Matron, 10);
			} else if (S.Matron.health.condition < 90) {
				improveCondition(S.Matron, 7);
			}

			if (S.Matron.devotion <= 60) {
				S.Matron.devotion++;
			}

			if (S.Matron.trust < 60) {
				S.Matron.trust++;
			}

			if (S.Matron.rules.living !== "luxurious") {
				S.Matron.rules.living = "luxurious";
			}
			if (S.Matron.rules.rest !== "restrictive") {
				S.Matron.rules.rest = "restrictive";
			}
		}
	}

	function matronText() {
		if (S.Matron) {
			const {He, he, His, his, him} = getPronouns(S.Matron);

			let r = [];

			r.push(`${SlaveFullName(S.Matron)} is serving as your Matron.`);

			if (S.Matron.relationship === -3 && S.Matron.devotion > 50) {
				r.push(`${He} does ${his} best to ${V.nurseryChildren ?
					`properly raise and look after the children in ${V.nurseryName}` :
					`look after ${V.nurseryName} and keep it tidy and running smoothly`}.`);
			}

			if (S.Matron.health.condition < 10) {
				r.push(`${He} does ${his} best to take good care of ${V.nurseryChildren ? `the children` : `${V.nurseryName}`}, but ${his} poor health limits ${his} ability to do so.`);
			} else if (S.Matron.health.condition < 80) {
				r.push(`${His} good health allows ${him} to keep up with the demands of ${V.nurseryName}${V.nurseryNannies ? `and <span class="yellowgreen">work</span> the nannies hard` : ``}.`);

				matronBonus += V.nurseryNannies ? 50 : 0;
			} else {
				r.push(`${His} perfect health allows ${him} to really stay on top of things in ${V.nurseryName}${V.nurseryNannies ? ` and <span class="yellowgreen">really drive</span> the nannies hard.` : ``}`);

				matronBonus += V.nurseryNannies ? 75 : 0;
			}

			if (App.Data.Careers.Leader.matron.includes(S.Matron.career)) {
				r.push(`${He} has experience with looking after children from ${his} life before ${he} was a slave.`);

				matronBonus += 25;
			} else if (S.Matron.skill.matron >= V.masteredXP) {
				r.push(`${He} has applicable experience with taking care of children from working for you.`);

				matronBonus += 25;
			} else {
				S.Matron.skill.matron += jsRandom(1, Math.ceil((
					S.Matron.intelligence + S.Matron.intelligenceImplant
				) / 15) + 8);
			}

			if (S.Matron.actualAge > 35) {
				r.push(`${His} age and experience make ${him} more effective.`);

				matronBonus += 25;
			} else if (!V.AgePenalty) {
				matronBonus += 25;
			}

			if (S.Matron.intelligence + S.Matron.intelligenceImplant > 15) {
				r.push(`${He}'s intelligent enough that ${he} is more effectively able to take care of ${V.nurseryName}${V.nurseryChildren ? ` and its children` : ``}.`);

				matronBonus += S.Matron.intelligence + S.Matron.intelligenceImplant;
			}

			if (V.nurseryChildren < V.nursery) {
				let seed = jsRandom(1, 10) + ((V.nursery - V.nurseryChildren) * (jsRandom(150, 170) + (matronBonus * 10)));
				cashX(seed, "nursery", S.Matron);

				r.push(`<p class="indent">Since ${he} doesn't have enough children to occupy all ${his} time, ${V.nurseryName} takes in citizens' children on a contract basis and ${he} cares for them too, earning <span class='yellowgreen'>${cashFormat(seed)}.</span></p>`);
			}

			if (arcology.FSRepopulationFocus > 0 && V.nurseryChildren > 0) {
				r.push(`Society <span class='green'>loves</span> the way you are raising more children for ${arcology.name}. `);
				FutureSocieties.Change("Repopulationist", 2);
			}
			return r.join(' ');
		}
	}

	const matronEffects = App.UI.DOM.appendNewElement("p", frag, '', "indent");

	matronChanges();
	$(matronEffects).append(matronText());

	if (slaves) {
		const intro = App.UI.DOM.appendNewElement("p", frag, '', "indent");

		let r = [];

		r.push(`${slaves.length > 1 ? `There are ${slaves.length} slaves` : `There is one slave`} working in ${V.nurseryName}.</strong>`);

		if (arcology.FSRepopulationFocus > 0 && V.nurseryChildren > 0) {
			r.push(`Society <span class="green">approves</span> of your bringing more children into this world.`);
		} else if (arcology.FSGenderFundamentalist > 0) {
			r.push(`Society <span class="green">approves</span> of your assigning slaves to a traditionally feminine role.`);
		}

		$(intro).append(r.join(' '));
	}

	if (S.Matron) {
		const slave = S.Matron;

		if (V.showEWD !== 0) {
			const matronEntry = App.UI.DOM.appendNewElement("div", frag, '', "slave-report");
			App.SlaveAssignment.appendSlaveArt(matronEntry, slave);
			matronEntry.append(App.UI.favoriteToggle(slave), " ");
			$(matronEntry).append(`<span class="slave-name">${SlaveFullName(slave)}</span> is serving as your Matron.`);
			matronEntry.append(App.SlaveAssignment.standardSlaveReport(slave, false));
		} else {
			App.SlaveAssignment.standardSlaveReport(slave, true);
		}
	}

	// FIXME: check these numbers over to make sure they make sense
	for (const slave of slaves) {
		slave.devotion += devBonus;

		if (slave.devotion <= 20 && slave.trust >= -20) {
			slave.devotion -= 5;
			slave.trust -= 5;
		} else if (slave.devotion <= 10) {
			slave.devotion += 2;
		} else if (slave.devotion >= 80) {
			slave.devotion -= 2;
		}

		if (slave.trust < -20) {
			slave.trust += 3;
		} else if (slave.trust < -50) {
			slave.trust += 2;
		} else if (slave.trust < -30) {
			slave.trust += 1;
		}

		if (slave.health.condition < -80) {
			improveCondition(slave, 20);
		} else if (slave.health.condition < -40) {
			improveCondition(slave, 10);
		} else if (slave.health.condition < 0) {
			improveCondition(slave, 7);
		} else if (slave.health.condition < 90) {
			improveCondition(slave, 3);
		}

		// TODO: add more cases
		switch (V.nurseryDecoration) {
			case "Repopulationist":
				slave.rules.living = "luxurious";
				break;
			case "Degradationist":
				slave.rules.living = "spare";
				break;
			default:
				slave.rules.living = "normal";
				break;
		}

		if (V.showEWD) {
			const {He} = getPronouns(slave);
			const slaveEntry = App.UI.DOM.appendNewElement("div", frag, '', "slave-report");
			App.SlaveAssignment.appendSlaveArt(slaveEntry, slave);
			slaveEntry.append(App.UI.favoriteToggle(slave), " ");
			$(slaveEntry).append(`<span class="slave-name">${SlaveFullName(slave)}</span> `);

			if (slave.choosesOwnAssignment === 2) {
				$(slaveEntry).append(App.SlaveAssignment.choosesOwnJob(slave));
			} else {
				$(slaveEntry).append(`is working out of ${V.nurseryName}.`);
			}

			const nannyContent = App.UI.DOM.appendNewElement("div", slaveEntry, '', "indent");

			$(nannyContent).append(`${He} ${App.SlaveAssignment.nanny(slave, matronBonus)}`);	// FIXME: nanny() takes one argument

			slaveEntry.append(App.SlaveAssignment.standardSlaveReport(slave, false));
		} else {	// silently discard return values
			App.SlaveAssignment.nanny(slave, matronBonus);	// FIXME: nanny() takes one argument
			App.SlaveAssignment.standardSlaveReport(slave, true);
		}
	}

	if (V.nurseryDecoration !== "standard") {
		const decorationEffects = App.UI.DOM.appendNewElement("p", frag, '', "indent");

		$(decorationEffects).append(`${capFirstChar(V.nurseryName)}'s ${V.nurseryDecoration} atmosphere <span class="devotion inc">has a minor impact on your servants.</span>`);
	}

	return frag;
};
