/**
 * Generates (and returns if not silent) a standard slave report
 * This is the part after the slave's job in most facilities.
 * @param {App.Entity.SlaveState} slave
 * @param {boolean} silent
 * @returns {DocumentFragment|null}
 */
App.SlaveAssignment.standardSlaveReport = function(slave, silent=false) {
	const clothes = App.SlaveAssignment.choosesOwnClothes(slave);
	const individualReport = App.SlaveAssignment.individualSlaveReport(slave);
	const devotion = App.SlaveAssignment.devotion(slave);

	if (!silent) {
		const container = document.createDocumentFragment();
		App.Events.addNode(container, [clothes, ...individualReport], "div", "indent");
		App.Events.addNode(container, [devotion], "div", "indent");
		return container;
	}
};

/**
 * Generates the main part of the standard slave report for an individual slave.
 * This is the section that's identical for all slaves regardless of facility.
 * @param {App.Entity.SlaveState} slave
 * @returns {Array<DocumentFragment|string>}
 */
App.SlaveAssignment.individualSlaveReport = function(slave) {
	return [
		App.SlaveAssignment.rules(slave),
		App.SlaveAssignment.diet(slave),
		App.SlaveAssignment.longTermEffects(slave),
		App.SlaveAssignment.drugs(slave),
		App.SlaveAssignment.relationships(slave),
		App.SlaveAssignment.rivalries(slave),
	];
};

/**
 * Render slave assignment report art
 * @param {ParentNode} node
 * @param {App.Entity.SlaveState} slave
 */
App.SlaveAssignment.appendSlaveArt = function(node, slave) {
	if (V.seeImages && V.seeReportImages) {
		App.UI.DOM.appendNewElement("div", node, App.EndWeek.saVars.slaveArt.render(slave), ["imageRef", "tinyImg"]);
	}
};
