App.UI.CustomSlaveTrade = function() {
	const frag = new DocumentFragment();
	let span = document.createElement("span");
	let baseControlsFilter = "all";
	App.UI.DOM.appendNewElement("p", frag, `When civilization turned upon itself, some countries readily took to enslaving their own. Others were raided by their neighbors for their desirable, and profitable, citizens. Which nationalities were most affected by the booming slave trade, and thus, likely to appear in your local slave markets?`);
	App.UI.DOM.appendNewElement("p", frag, `+ to increase pop. – to reduce pop. 0 to remove entirely`, "bold");

	span.id = "pop-control";
	span.append(baseNationalitiesControls());
	frag.append(span);

	return frag;

	function baseNationalitiesControls() {
		const frag = new DocumentFragment();
		let p = document.createElement("p");
		let destination;
		let div;
		let array;
		let list;
		let span;
		const nationalitiesCheck = App.UI.nationalitiesCheck();

		if (hashSum(V.nationalities) < 1) {
			App.UI.DOM.appendNewElement("div", p, `You cannot be a slaveowner without a slave trade. Please add nationalities to continue.`, "note");
		} else {
			destination = (V.customWA === 0) ? "Intro Summary" : "Extreme Intro";
			App.UI.DOM.appendNewElement(
				"div",
				p,
				App.UI.DOM.passageLink(
					"Confirm customization",
					destination,
					() => baseControlsFilter = undefined
				)
			);
		}
		frag.append(p);

		App.UI.DOM.appendNewElement("p", frag, App.UI.nationalitiesDisplay());

		p = document.createElement("p");

		/* Fine control tweaking of populations */
		App.UI.DOM.appendNewElement("div", p, `Adjust slave populations:`);

		p.append(sectionBreak());

		/* Filter controls */
		div = document.createElement("div");
		App.UI.DOM.appendNewElement("span", div, `Filter by Race: `);
		array = [];
		for (const race of setup.filterRaces) {
			if (baseControlsFilter === uncapFirstChar(race).replace(/[ -]/g, '')) {
				array.push(
					App.UI.DOM.disabledLink(
						race,
						["currently selected race"]
					)
				);
			} else {
				array.push(
					App.UI.DOM.link(
						race,
						() => {
							baseControlsFilter = uncapFirstChar(race).replace(/[ -]/g, '');
							refresh();
						}
					)
				);
			}
		}
		div.append(App.UI.DOM.generateLinksStrip(array));
		p.append(div);

		div = document.createElement("div");
		App.UI.DOM.appendNewElement("span", div, `Filter by Region: `);
		array = [];
		for (const region of setup.filterRegions) {
			if (baseControlsFilter === uncapFirstChar(region).replace(/[ -]/g, '')) {
				array.push(
					App.UI.DOM.disabledLink(
						region,
						["currently selected region"]
					)
				);
			} else {
				array.push(
					App.UI.DOM.link(
						region,
						() => {
							baseControlsFilter = uncapFirstChar(region).replace(/[ -]/g, '');
							refresh();
						}
					)
				);
			}
		}
		div.append(App.UI.DOM.generateLinksStrip(array));
		p.append(div);

		p.append(sectionBreak());

		div = document.createElement("div");
		div.style.whiteSpace = "nowrap";
		div.style.float = "left";

		/* Unfiltered pop controls */
		list = document.createElement("UL");
		list.classList.add("customize-slave-trade-ul");
		if (baseControlsFilter === "all") {
			for (const nation of setup.baseNationalities) {
				const li = document.createElement("LI");
				li.classList.add("customize-slave-trade-li");
				li.append(nation);

				span = document.createElement("span");
				span.classList.add("customize-slave-trade-li-container");
				App.UI.DOM.appendNewElement(
					"span",
					span,
					App.UI.DOM.link(
						`+`,
						() => {
							hashPush(V.nationalities, nation);
							refresh();
						}
					),
					"plusButton"
				);
				if (nationalitiesCheck[nation]) {
					App.UI.DOM.appendNewElement(
						"span",
						span,
						App.UI.DOM.link(
							`-`,
							() => {
								V.nationalities[nation] -= 1;
								if (V.nationalities[nation] <= 0) {
									delete V.nationalities[nation];
								}
								refresh();
							}
						),
						"minusButton"
					);
				}
				if (V.nationalities[nation] > 1) {
					App.UI.DOM.appendNewElement(
						"span",
						span,
						App.UI.DOM.link(
							`0`,
							() => {
								delete V.nationalities[nation];
								refresh();
							}
						),
						"zeroButton"
					);
				}
				li.append(span);
				list.append(li);
			}
			p.append(list);
			App.UI.DOM.appendNewElement("div", p, `By dominant race/ethnicity (hover over the name to see the nationalities affected):`);
			list = document.createElement("UL");
			list.classList.add("customize-slave-trade-ul");
			for (const race of setup.filterRaces) {
				const racialNationalities = setup.baseNationalities.filter(function(n) {
					let races = setup.raceSelector[n] || setup.raceSelector[''];
					return races[race.toLowerCase()] * 3.5 > hashSum(races);
				});

				if (racialNationalities.length > 0) {
					const li = document.createElement("LI");
					li.classList.add("customize-slave-trade-li");
					li.title = racialNationalities.length > 0 ? racialNationalities.join(", ") : "(none)";
					li.append(race);
					span = document.createElement("span");
					span.classList.add("customize-slave-trade-li-container");
					App.UI.DOM.appendNewElement(
						"span",
						span,
						App.UI.DOM.link(
							`+`,
							() => {
								racialNationalities.forEach(n => hashPush(V.nationalities, n));
								refresh();
							}
						),
						"plusButton"
					);
					App.UI.DOM.appendNewElement(
						"span",
						span,
						App.UI.DOM.link(
							`0`,
							() => {
								racialNationalities.forEach(n => delete V.nationalities[n]);
								refresh();
							}
						),
						"zeroButton"
					);
					li.append(span);
					list.append(li);
				}
			}
		} else {
			/* Filtered pop controls */
			const controlsNationality = setup[baseControlsFilter + 'Nationalities'];
			const keys = Object.keys(controlsNationality);
			for (const nation of keys) {
				const li = document.createElement("LI");
				li.classList.add("customize-slave-trade-li");
				li.append(nation);
				span = document.createElement("span");
				span.classList.add("customize-slave-trade-li-container");
				App.UI.DOM.appendNewElement(
					"span",
					span,
					App.UI.DOM.link(
						`+`,
						() => {
							hashPush(V.nationalities, nation);
							refresh();
						}
					),
					"plusButton"
				);

				if (nationalitiesCheck[nation]) {
					App.UI.DOM.appendNewElement(
						"span",
						span,
						App.UI.DOM.link(
							`-`,
							() => {
								V.nationalities[nation] -= 1;
								if (V.nationalities[nation] <= 0) {
									delete V.nationalities[nation];
								}
								refresh();
							}
						),
						"minusButton"
					);
				}
				if (V.nationalities[nation] > 1) {
					App.UI.DOM.appendNewElement(
						"span",
						span,
						App.UI.DOM.link(
							`0`,
							() => {
								delete V.nationalities[nation];
								refresh();
							}
						),
						"zeroButton"
					);
				}
				li.append(span);
				list.append(li);
			}
		}
		p.append(list);
		frag.append(p);

		div = document.createElement("div");

		div.append(
			App.UI.DOM.link(
				"Reset filters",
				() => {
					baseControlsFilter = "all";
					refresh();
				}
			)
		);

		div.append(" | ");

		div.append(
			App.UI.DOM.link(
				"Clear all nationalities",
				() => {
					V.nationalities = {};
					refresh();
				}
			)
		);

		frag.append(div);

		frag.append(sectionBreak());

		div = document.createElement("div");
		div.append(`Vanilla presets: `);
		div.append(generatePresetLinks("Vanilla"));
		frag.append(div);

		div = document.createElement("div");
		div.append(`Mod presets: `);
		div.append(generatePresetLinks("Mod"));
		frag.append(div);

		p = document.createElement("p");
		App.UI.DOM.appendNewElement(
			"span",
			p,
			App.UI.DOM.link(
				"Export Settings",
				() => {
					settingsExport();
				}
			)
		);
		p.append(" | ");
		App.UI.DOM.appendNewElement(
			"span",
			p,
			App.UI.DOM.link(
				"Import Settings",
				() => {
					settingsImport();
				}
			)
		);
		frag.append(p);
		div = document.createElement("div");
		div.id = "importExportArea";
		frag.append(div);

		return frag;
	}

	function sectionBreak() {
		const hr = document.createElement("hr");
		hr.style.margin = "0";
		return hr;
	}

	function refresh() {
		return jQuery('#pop-control').empty().append(baseNationalitiesControls());
	}

	function settingsExport() {
		let textArea = document.createElement("textarea");
		textArea.value = JSON.stringify(V.nationalities);
		$("#importExportArea").html(textArea);
	}

	function settingsImport() {
		let textArea = document.createElement("textarea");
		let button = document.createElement("button");
		button.name = "Load";
		button.innerHTML = "Load";
		button.onclick = () => {
			V.nationalities = JSON.parse(textArea.value);
			State.display(State.passage);
		};

		$("#importExportArea").html("").append(textArea).append(button);
	}

	function generatePresetLinks(group) {
		let links = [];
		for (const [name, nationalities] of App.Data.NationalityPresets[group]) {
			links.push(
				App.UI.DOM.link(
					name,
					() => {
						V.nationalities = clone(nationalities);
						refresh();
					}
				)
			);
		}
		return App.UI.DOM.generateLinksStrip(links);
	}
};

/**
 * @returns {HTMLElement}
 */
App.UI.nationalitiesDisplay = function() {
	const p = document.createElement("p");

	/* Generates cloned array of V.nationalities, removing duplicates and then sorting */
	const nationalitiesCheck = App.UI.nationalitiesCheck();

	/* Prints distribution of V.nationalities, using nationalitiesCheck to render array */
	let percentPerPoint = 100.0 / hashSum(V.nationalities);
	let len = Object.keys(nationalitiesCheck).length;
	let j = 0;
	for (const nation in nationalitiesCheck) {
		const span = document.createElement("span");
		span.append(`${nation} `);
		App.UI.DOM.appendNewElement("span", span, (V.nationalities[nation] * percentPerPoint).toFixed(2), "orange");
		j++;
		if (j < len) {
			span.append(` | `);
		}
		p.append(span);
	}
	return p;
};

App.UI.nationalitiesCheck = function() {
	return Object.assign(
		{
			// Player can add custom nations here.
		},
		V.nationalities);
};
