App.Intro.summary = function() {
	const el = new DocumentFragment();

	V.neighboringArcologies = variableAsNumber(V.neighboringArcologies, 0, 8, 3);
	V.FSCreditCount = variableAsNumber(V.FSCreditCount, 4, 7, 5);
	V.PC.actualAge = variableAsNumber(V.PC.actualAge, 14, 80, 35);
	V.PC.birthWeek = variableAsNumber(V.PC.birthWeek, 0, 51, 0);

	el.append(introContent());

	V.minimumSlaveAge = variableAsNumber(V.minimumSlaveAge, 3, 18, 18);
	V.retirementAge = variableAsNumber(V.retirementAge, 25, 120, 45);
	V.fertilityAge = variableAsNumber(V.fertilityAge, 3, 18, 13);
	V.potencyAge = variableAsNumber(V.potencyAge, 3, 18, 13);
	V.PC.mother = Number(V.PC.mother);
	V.PC.father = Number(V.PC.father);
	if (V.freshPC === 1 || V.saveImported === 0) {
		V.PC.origRace = V.PC.race;
		V.PC.origSkin = V.PC.skin;
		V.PC.origEye = V.PC.eye.right.iris; // needed for compatibility
		V.PC.origHColor = V.PC.hColor;
	}

	App.UI.tabBar.handlePreSelectedTab(V.tabChoice.IntroSummary);
	/**
	 * @typedef {Object} siCategory
	 * @property {string} title
	 * @property {string} id
	 * @property {DocumentFragment|HTMLElement} node
	 * @property {Function} [onClick]
	 */

	/** @type {Array<siCategory>} */
	const buttons = [
		{
			title: "World",
			id: "world",
			get node() { return worldContent(); }
		},
		{
			title: "Slaves",
			id: "slaves",
			get node() { return slavesContent(); }
		},
		{
			title: "Player Character",
			id: "player",
			get node() { return App.UI.Player.design(); }
		},
		{
			title: "UI",
			id: "interface",
			get node() { return interfaceContent(); }
		}
	];

	el.append(displayWithTabs());

	return el;

	function displayWithTabs() {
		const el = new DocumentFragment();

		const tabBar = document.createElement("div");
		tabBar.classList.add("tab-bar");
		const tabContents = new DocumentFragment();

		for (const item of buttons) {
			tabBar.append(makeTabButton(item));
			tabContents.append(makeTabContents(item));
		}

		el.append(tabBar);

		el.append(tabContents);

		return el;

		/**
		 * @param {siCategory} item
		 * @returns {HTMLElement}
		 */
		function makeTabButton(item) {
			const btn = document.createElement("button");
			btn.className = "tab-links";
			btn.id = `tab ${item.id}`;
			btn.innerHTML = item.title;
			btn.onclick = (event) => {
				App.UI.tabBar.openTab(event, item.id);
				jQuery(`#content-${item.id}`).empty().append(item.node);
				if (item.hasOwnProperty("onClick")){
					item.onClick();
				}
			};

			return btn;
		}
	}

	/**
	 * @param {siCategory} item
	 * @returns {HTMLElement}
	 */
	function makeTabContents(item) {
		const wrapperEl = document.createElement("div");
		wrapperEl.id = item.id;
		wrapperEl.classList.add("tab-content");

		const classEl = document.createElement("div");
		classEl.classList.add("content");

		classEl.append(makeContentSpan(item));
		wrapperEl.append(classEl);

		return wrapperEl;
	}

	/**
	 * @param {siCategory} item
	 * @returns {HTMLElement}
	 */
	function makeContentSpan(item) {
		const uniqueContent = document.createElement("span");
		uniqueContent.id = `content-${item.id}`;

		uniqueContent.append(item.node);
		return uniqueContent;
	}

	function introContent() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("div", el, `You may review your settings before clicking "Continue" to begin.`);

		const linkArray = [];
		linkArray.push(
			App.UI.DOM.link(
				"Continue",
				() => {
					if (V.freshPC === 1 || V.saveImported === 0) {
						switch (V.PC.career) {
							case "arcology owner":
								V.PC.skill.trading = 100;
								V.PC.skill.warfare = 100;
								V.PC.skill.hacking = 100;
								V.PC.skill.slaving = 100;
								V.PC.skill.engineering = 100;
								V.PC.skill.medicine = 100;
								break;
							case "wealth":
								if (V.PC.vagina === 1) {
									V.PC.vagina = 2;
								}
								V.PC.weight = 60;
								V.PC.muscles = 0;
								break;
							case "trust fund":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.warfare = -50;
								V.PC.skill.slaving = -50;
								V.PC.skill.engineering = -50;
								V.PC.skill.medicine = -50;
								V.PC.weight = 60;
								V.PC.muscles = 0;
								break;
							case "rich kid":
								V.PC.intelligenceImplant = 5;
								V.PC.skill.trading = -25;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -25;
								V.PC.weight = 60;
								V.PC.muscles = 0;
								break;
							case "capitalist":
								V.PC.skill.trading = 100;
								V.PC.muscles = 0;
								break;
							case "entrepreneur":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = 50;
								V.PC.skill.warfare = -25;
								V.PC.skill.slaving = -25;
								V.PC.skill.engineering = -25;
								V.PC.skill.medicine = -25;
								V.PC.muscles = 0;
								break;
							case "business kid":
								V.PC.intelligenceImplant = 5;
								V.PC.skill.warfare = -80;
								V.PC.skill.slaving = -80;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -20;
								V.PC.muscles = 0;
								break;
							case "mercenary":
								V.PC.skill.warfare = 100;
								V.PC.muscles = 50;
								break;
							case "recruit":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = -25;
								V.PC.skill.warfare = 50;
								V.PC.skill.slaving = -25;
								V.PC.skill.engineering = -25;
								V.PC.skill.medicine = -25;
								V.PC.muscles = 40;
								break;
							case "child soldier":
								V.PC.intelligenceImplant = 0;
								V.PC.skill.trading = -100;
								V.PC.skill.slaving = -80;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -80;
								break;
							case "slaver":
								V.PC.skill.slaving = 100;
								V.PC.muscles = 50;
								break;
							case "slave overseer":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = -20;
								V.PC.skill.warfare = -20;
								V.PC.skill.slaving = 50;
								V.PC.skill.engineering = -25;
								V.PC.skill.medicine = -20;
								V.PC.muscles = 50;
								break;
							case "slave tender":
								V.PC.intelligenceImplant = 0;
								V.PC.skill.trading = -100;
								V.PC.skill.warfare = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -60;
								V.PC.skill.hacking = -100;
								V.PC.muscles = 10;
								break;
							case "engineer":
								V.PC.skill.engineering = 100;
								break;
							case "construction":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = -25;
								V.PC.skill.warfare = -50;
								V.PC.skill.slaving = -25;
								V.PC.skill.engineering = 50;
								V.PC.skill.medicine = -25;
								V.PC.skill.hacking = -20;
								V.PC.muscles = 50;
								break;
							case "worksite helper":
								V.PC.intelligenceImplant = 0;
								V.PC.skill.trading = -80;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = 0;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -100;
								break;
							case "medicine":
								V.PC.skill.medicine = 100;
								V.PC.muscles = 0;
								break;
							case "medical assistant":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = -25;
								V.PC.skill.warfare = -50;
								V.PC.skill.slaving = -25;
								V.PC.skill.engineering = 50;
								V.PC.skill.medicine = -25;
								V.PC.skill.hacking = -20;
								V.PC.muscles = 0;
								break;
							case "nurse":
								V.PC.intelligenceImplant = 5;
								V.PC.skill.trading = -100;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.hacking = -20;
								V.PC.muscles = 0;
								break;
							case "celebrity":
								if (V.PC.vagina === 1) {
									V.PC.vagina = 2;
								}
								V.PC.muscles = -20;
								break;
							case "rising star":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = -50;
								V.PC.skill.warfare = -50;
								V.PC.skill.slaving = -50;
								V.PC.skill.engineering = -50;
								V.PC.skill.medicine = -50;
								V.PC.muscles = -20;
								break;
							case "child star":
								V.PC.intelligenceImplant = 0;
								V.PC.skill.trading = -100;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -20;
								V.PC.muscles = 0;
								break;
							case "BlackHat":
								V.PC.skill.hacking = 100;
								V.PC.muscles = -20;
								break;
							case "hacker":
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = -50;
								V.PC.skill.warfare = -50;
								V.PC.skill.slaving = -50;
								V.PC.skill.engineering = -50;
								V.PC.skill.medicine = -50;
								V.PC.skill.hacking = 50;
								V.PC.muscles = -20;
								break;
							case "script kiddy":
								V.PC.intelligenceImplant = 5;
								V.PC.skill.trading = -80;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -80;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = 20;
								V.PC.muscles = -20;
								break;
							case "escort":
								if (V.PC.vagina >= 0) {
									V.PC.vagina = 4;
								}
								V.PC.anus = 1;
								V.PC.clothes = "a slutty outfit";
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = 50;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = 10;
								V.PC.skill.hacking = 10;
								V.PC.muscles = 0;
								break;
							case "prostitute":
								if (V.PC.vagina >= 0) {
									V.PC.vagina = 3;
								}
								V.PC.anus = 1;
								V.PC.clothes = "a slutty outfit";
								V.PC.intelligenceImplant = 0;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -50;
								V.PC.skill.hacking = -20;
								V.PC.muscles = 0;
								break;
							case "child prostitute":
								if (V.PC.vagina >= 0) {
									V.PC.vagina = 2;
								}
								V.PC.anus = 1;
								V.PC.clothes = "a slutty outfit";
								V.PC.intelligenceImplant = 0;
								V.PC.skill.trading = -50;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -80;
								V.PC.muscles = -20;
								break;
							case "servant":
								V.PC.clothes = "a nice maid outfit";
								V.PC.intelligenceImplant = 0;
								if (V.PC.vagina >= 1) {
									V.PC.vagina = 4;
								}
								if (V.PC.vagina >= 0) {
									V.PC.geneticQuirks.fertility = 2;
								}
								V.PC.skill.trading = -100;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -100;
								V.PC.muscles = 0;
								V.PC.geneticQuirks.fertility = 1;
								break;
							case "handmaiden":
								V.PC.clothes = "a nice maid outfit";
								V.PC.intelligenceImplant = 0;
								if (V.PC.vagina >= 1) {
									V.PC.vagina = 3;
								}
								if (V.PC.vagina >= 0) {
									V.PC.geneticQuirks.fertility = 2;
								}
								V.PC.skill.trading = -100;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -100;
								V.PC.muscles = 0;
								V.PC.geneticQuirks.fertility = 1;
								break;
							case "child servant":
								V.PC.clothes = "a nice maid outfit";
								V.PC.intelligenceImplant = 0;
								if (V.PC.vagina >= 1) {
									V.PC.vagina = 2;
								}
								if (V.PC.vagina >= 0) {
									V.PC.geneticQuirks.fertility = 2;
								}
								V.PC.skill.trading = -100;
								V.PC.skill.warfare = -100;
								V.PC.skill.slaving = -100;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -100;
								V.PC.muscles = 0;
								V.PC.geneticQuirks.fertility = 1;
								break;
							case "gang":
								if (V.PC.vagina === 1) {
									V.PC.vagina = 2;
								}
								V.PC.intelligenceImplant = 15;
								V.PC.skill.trading = 50;
								V.PC.skill.warfare = 50;
								V.PC.skill.slaving = 50;
								V.PC.skill.engineering = -100;
								V.PC.skill.hacking = 50;
								V.PC.muscles = 60;
								break;
							case "hoodlum":
								V.PC.intelligenceImplant = 0;
								V.PC.skill.warfare = -20;
								V.PC.skill.slaving = -20;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -50;
								V.PC.skill.hacking = 0;
								break;
							case "street urchin":
								V.PC.intelligenceImplant = 0;
								V.PC.skill.trading = -20;
								V.PC.skill.warfare = -40;
								V.PC.skill.slaving = -80;
								V.PC.skill.engineering = -100;
								V.PC.skill.medicine = -100;
								V.PC.skill.hacking = -100;
								break;
						}
						if (V.PC.rumor === "diligence") {
							V.PC.weight = 0;
							if (V.PC.muscles < 30) {
								V.PC.muscles += 20;
							}
						} else if (V.PC.rumor === "force") {
							V.PC.muscles += 20;
						}
					}

					if (V.PC.dick !== 0) {
						V.PC.geneticQuirks.wellHung = 2;
					} else {
						V.PC.balls = 0;
						V.PC.scrotum = 0;
						V.PC.prostate = 0;
					}
					if (V.PC.title === 0) {
						V.PC.hLength = 15;
						V.PC.waist = -20;
						V.PC.voice = 2;
						V.PC.shoulders = -1;
						V.PC.hips = 1;
					}
					if (V.PC.vagina === -1) {
						V.PC.ovaries = 0;
					} else if (V.PC.vagina > 0) {
						V.PC.vaginaLube = 1;
					}
					V.PC.ovaryAge = V.PC.physicalAge;

					if (V.saveImported === 1 && V.freshPC === 0 && V.PC.rules.living !== "luxurious") {
						if (V.PC.rules.living === "spare") {
							V.PC.rules.living = "normal";
						} else {
							V.PC.rules.living = "luxurious";
						}
					} else if (["wealth", "trust fund", "rich kid", "celebrity", "rising star", "child star"].includes(V.PC.career)) {
						V.PC.rules.living = "normal";
					} else {
						V.PC.rules.living = "spare";
					}
					App.Intro.initNationalities();
					SectorCounts(); // Update AProsperityCap
				},
				[],
				"Starting Girls"
			)
		);

		if ((V.economy !== 100) || (V.seeDicks !== 25) || (V.continent !== "North America") || (V.internationalTrade !== 1) || (V.internationalVariety !== 1) || (V.seeRace !== 1) || (V.seeNationality !== 1) || (V.seeExtreme !== 0) || (V.seeCircumcision !== 1) || (V.seeAge !== 1) || (V.plot !== 1)) {
			linkArray.push(
				App.UI.DOM.link(
					"restore defaults",
					() => {
						V.seeDicks = 25;
						V.economy = 100;
						V.continent = "North America";
						V.internationalTrade = 1;
						V.internationalVariety = 1;
						V.seeRace = 1;
						V.seeNationality = 1;
						V.seeExtreme = 0;
						V.seeCircumcision = 1;
						V.seeAge = 1;
						V.plot = 1;
					},
					[],
					"Intro Summary"
				)
			);
		}
		linkArray.push(
			App.UI.DOM.link(
				"Cheat Start",
				() => {
					cashX(1000000, "cheating");
					V.PC.rules.living = "luxurious";
					repX(20000, "cheating");
					V.dojo += 1;
					V.cheatMode = 1;
					V.seeDesk = 0;
					V.seeFCNN = 0;
					V.sortSlavesBy = "devotion";
					V.sortSlavesOrder = "descending";
					V.sortSlavesMain = 0;
					V.rulesAssistantMain = 1;
					Object.assign(
						V.UI.slaveSummary.abbreviation,
						{
							devotion: 1,
							rules: 1,
							clothes: 2,
							health: 1,
							diet: 1,
							drugs: 1,
							race: 1,
							nationality: 1,
							genitalia: 1,
							physicals: 1,
							skills: 1,
							mental: 2
						}
					);
					V.PC.skill.trading = 100;
					V.PC.skill.warfare = 100;
					V.PC.skill.slaving = 100;
					V.PC.skill.engineering = 100;
					V.PC.skill.medicine = 100;
					V.PC.skill.hacking = 100;
					App.Intro.initNationalities();
				},
				[],
				"Starting Girls",
				"Intended for debugging: may have unexpected effects"
			)
		);
		App.UI.DOM.appendNewElement("div", el, App.UI.DOM.generateLinksStrip(linkArray));

		return el;
	}

	function worldContent() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h2", el, "Economy");

		let options = new App.UI.OptionsGroup();

		V.localEcon = V.economy;
		options.addOption("Economic climate", "baseDifficulty")
			.addValue("Very Easy", 1, () => V.economy = 200)
			.addValue("Easy", 2, () => V.economy = 125)
			.addValue("Default Difficulty", 3, () => V.economy = 100)
			.addValue("Hard", 4, () => V.economy = 80)
			.addValue("Very Hard", 5, () => V.economy = 67);

		if (V.difficultySwitch === 1) {
			V.econAdvantage = -2;
		}
		if (V.difficultySwitch === 0) {
			V.econRate = 0;
		}
		options.addOption("Economic forecast", "econRate").addComment("Some economic content requires this to be set to harder than vanilla")
			.addValue("Vanilla", 0, () => V.difficultySwitch = 0)
			.addValue("Easy", 1, () => V.difficultySwitch = 1)
			.addValue("Default", 2, () => V.difficultySwitch = 1)
			.addValue("Hard", 3, () => V.difficultySwitch = 1);

		/* Not functional yet
		All the things you need to run your arcology are getting more expensive
		if (V.incomeMod === 0) {
			while all forms of income <strong>remain static</strong>.
				[[Easier|Intro Summary][V.incomeMod = 1]]
		} else if (V.incomeMod === 1) {
			while all forms of income <strong>rise but cannot keep pace</strong>.
			[[Harder|Intro Summary][V.incomeMod = 0]] | [[Easier|Intro Summary][V.incomeMod = 2]]
		} else {
			but luckily all forms of income <strong>rise in lockstep</strong>.
			[[Harder|Intro Summary][V.incomeMod = 1]]
		}
		*/

		V.foodCost = Math.trunc(2500 / V.economy);
		V.drugsCost = Math.trunc(10000 / V.economy);
		V.rulesCost = Math.trunc(10000 / V.economy);
		V.modCost = Math.trunc(5000 / V.economy);
		V.surgeryCost = Math.trunc(30000 / V.economy);

		if (!V.customVariety) {
			options.addOption("You are using standardized slave trading channels.")
				.customButton("Customize the slave trade", () => { V.customVariety = 1; V.customWA = 0; }, "Customize Slave Trade");

			options.addOption("", "internationalTrade")
				.addValue("Allow intercontinental trade", 1).on().customDescription("The slave trade is <strong>international,</strong> so a wider variety of slaves will be available.")
				.addValue("Restrict the trade to continental", 0).off().customDescription("The slave trade is <strong>continental,</strong> so a narrower variety of slaves will be available.");

			if (V.internationalTrade === 1) {
				options.addOption("International slave variety is", "internationalVariety")
					.addValue("Normalized national variety", 1).customDescription("<strong>normalized,</strong> so small nations will appear nearly as much as large ones.")
					.addValue("Semi-realistic national variety", 0).customDescription("<strong>semi-realistic,</strong> so more populous nations will be more common.");
			}
		} else {
			options.addOption("Nationality distributions are customized.")
				.customButton("Adjust the slave trade", () => { V.customWA = 0; V.customVariety = 1; }, "Customize Slave Trade")
				.customButton("Stop customizing", () => { delete V.customVariety; });
		}
		/* closes V.customVariety is defined */

		if (V.customVariety) {
			options.addCustom(App.UI.nationalitiesDisplay());
		}

		options.addOption("", "plot")
			.addValue("Enable non-erotic events", 1).on().customDescription("Game mode: <strong>two-handed</strong>. Includes non-erotic events concerning the changing world.")
			.addValue("Disable non-erotic events", 0).off().customDescription("Game mode: <strong>one-handed</strong>. No non-erotic events concerning the changing world.");

		el.append(options.render());


		App.UI.DOM.appendNewElement("h2", el, "The Free City");
		options = new App.UI.OptionsGroup();

		options.addOption(`The Free City features <strong>${V.neighboringArcologies}</strong> arcologies in addition to your own.`, "neighboringArcologies")
			.showTextBox().addComment("Setting this to 0 will disable most content involving the rest of the Free City.");

		if (V.targetArcology.fs === "New") {
			options.addOption(`The Free City is located on <strong>${V.terrain}</strong> terrain.`, "terrain")
				.addValueList([
					["Urban", "urban"],
					["Rural", "rural"],
					["Ravine", "ravine"],
					["Marine", "marine"],
					["Oceanic", "oceanic"]
				]);

			if (V.terrain !== "oceanic") {
				// TODO: this is from original, but seems unused?
				options.addOption(`The Free City is located in <strong>${V.continent}</strong>.`, "continent")
					.addValue("North America").addCallback(() => V.language = "English")
					.addValue("South America").addCallback(() => V.language = "Spanish")
					.addValue("Brazil").addCallback(() => V.language = "Portuguese")
					.addValue("Western Europe").addCallback(() => V.language = "English")
					.addValue("Central Europe").addCallback(() => V.language = "German")
					.addValue("Eastern Europe").addCallback(() => V.language = "Russian")
					.addValue("Southern Europe").addCallback(() => V.language = "Italian")
					.addValue("Scandinavia").addCallback(() => V.language = "Norwegian")
					.addValue("the Middle East").addCallback(() => V.language = "Arabic")
					.addValue("Africa").addCallback(() => V.language = "Arabic")
					.addValue("Asia").addCallback(() => V.language = "Chinese")
					.addValue("Australia").addCallback(() => V.language = "English")
					.addValue("Japan").addCallback(() => V.language = "Japanese")
					.pulldown();
			}

			options.addOption("The lingua franca of your arcology is", "language")
				.showTextBox();
		} else if (!["ArabianRevivalist", "AztecRevivalist", "ChineseRevivalist", "EdoRevivalist", "EgyptianRevivalist", "RomanRevivalist"].includes(V.targetArcology.fs)) {
			options.addOption("The lingua franca of your arcology is", "language")
				.addValueList(["English", "Spanish", "Arabic"]).showTextBox();
		}

		options.addOption(`The Free City could develop as many as <strong>${V.FSCreditCount}</strong> future societies.`, "FSCreditCount")
			.showTextBox().addComment("5 is default, 4 behaves the same as pre-patch 0.9.9.0, max is 7. This option cannot be changed during the game.");

		el.append(options.render());

		App.UI.DOM.appendNewElement("h2", el, "Content");
		options = new App.UI.OptionsGroup();

		options.addOption("Proportion of slave girls with dicks", "seeDicks")
			.addValueList([
				["None (0%)", 0],
				["Nearly none (1%)", 1],
				["A few (10%)", 10],
				["Some (25%)", 25],
				["Half (50%)", 50],
				["Lots (75%)", 75],
				["Most (90%)", 90],
				["Almost all (99%)", 99],
				["All (100%)", 100]
			]);

		if (V.seeDicks === 0) {
			options.addOption("Should you be able to surgically attach a penis to your female slaves and starting girls?", "makeDicks")
				.addValue("Yes", 1).on().addValue("No", 0).off();
		}

		options.addOption("Slaves getting sick is", "seeIllness")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Pregnancy related content is", "seePreg")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Children born in game strictly adhering to dick content settings is", "seeDicksAffectsPregnancy")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		if (V.seeDicksAffectsPregnancy === 0) {
			options.addOption("XX slaves only fathering daughters is", "adamPrinciple")
				.addValue("Enabled", 1).on().addValue("Disabled", 0).off();
		}

		options.addOption("Extreme pregnancy content like broodmothers is", "seeHyperPreg")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Advanced pregnancy complications such as miscarriage and premature birth are", "dangerousPregnancy")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Extreme content like amputation is", "seeExtreme")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Bestiality content is", "seeBestiality")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Watersports content is", "seePee")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Incest is", "seeIncest")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();


		if (V.seeDicks !== 0 || V.makeDicks !== 0) {
			options.addOption("Circumcision is", "seeCircumcision")
				.addValue("Enabled", 1).on().addValue("Disabled", 0).off();
		}

		el.append(options.render());

		App.UI.DOM.appendNewElement("h2", el, "Mods");

		options = new App.UI.OptionsGroup();

		options.addOption("The Special Force Mod is", "Toggle", V.SF)
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();
		options.addComment(`This mod is initially from anon1888 but expanded by SFanon offers a lategame special (started out as security but changed to special in order to try and reduce confusion with CrimeAnon's separate Security Expansion (SecExp) mod) force, that is triggered after week 72.
		It is non-canon where it conflicts with canonical updates to the base game.`);

		options.addOption("The Security Expansion Mod is", "secExpEnabled")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();
		options.addComment(`This mod introduces security and crime in the arcology, as well as attacks and battles.
		The mod can be activated in any moment, but it may result in unbalanced gameplay if activated very late in the game.`);

		el.append(options.render());
		return el;
	}

	function slavesContent() {
		const el = new DocumentFragment();
		let options = new App.UI.OptionsGroup();

		options.addOption("Master Suite report details such as slave changes are", "verboseDescriptions")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Slaves can have alternate titles", "newDescriptions")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Mention ethnicity", "seeRace")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Mention nationality", "seeNationality")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Dynasties of enslaved royalties are", "realRoyalties")
			.addValueList([
				["Historical", 1],
				["Random", 0]
			]);

		options.addOption("New slaves may have male names", "allowMaleSlaveNames").addComment("This only affects slave generation and not your ability to name your slaves.")
			.addValue("Enabled", true).on().addValue("Disabled", false).off();

		options.addOption("Schema for ordering slave names is", "surnameOrder")
			.addValueList([
				["Country of origin", 0],
				["Name Surname", 1],
				["Surname Name", 2]
			]);

		options.addOption("Family size", "limitFamilies").addComment("Controls acquisition of additional relatives, by means other than birth, for slaves with families.")
			.addValue("Allow extended families", 0).on().addValue("Restrict family size (Vanilla Emulation)", 1).off();

		options.addOption("Tracking distant relatives is", "showDistantRelatives")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Successive breeding resulting in sub-average slaves is", "inbreeding")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Family titles for relatives", "allowFamilyTitles")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Slave assets affected by weight is", "weightAffectsAssets")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off()
			.addComment("Diet will still affect asset size.");

		options.addOption("Curative side effects are", "curativeSideEffects")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Slaves with fat lips or heavy oral piercings may lisp", "disableLisping")
			.addValue("Yes", 0).on().addValue("No", 1).off();

		el.append(options.render());

		App.UI.DOM.appendNewElement("h2", el, "Age");
		options = new App.UI.OptionsGroup();

		options.addOption("Slave aging", "seeAge")
			.addValue("Enabled", 1).on().addValue("Celebrate birthdays, but don't age.", 2).neutral().addValue("Disabled", 0).off();

		options.addOption("Slave age distribution", "pedo_mode").addComment("In loli mode most new slaves are under the age of 18. May not apply to custom slaves and slaves from specific events.")
			.addValue("Loli mode", 1, () => V.minimumSlaveAge = 5).addValue("Normal mode", 0);

		V.minimumSlaveAge = Math.clamp(V.minimumSlaveAge, 3, 18);
		options.addOption("Girls appearing in the game will be no younger than", "minimumSlaveAge").showTextBox();

		options.addOption(`Molestation of slaves younger than ${V.minimumSlaveAge} is`, "extremeUnderage")
			.addValue("Permitted", 1).on().addValue("Forbidden", 0).off();

		V.retirementAge = Math.clamp(V.retirementAge, V.minimumSlaveAge + 1, 120);
		options.addOption("Initial retirement age will be at", "retirementAge")
			.addComment("May cause issues with New Game and initial slaves if set below 45.").showTextBox();

		V.fertilityAge = Math.clamp(V.fertilityAge, 3, 18);
		options.addOption("Girls will not be able to become pregnant if their age is under", "fertilityAge").showTextBox();

		V.potencyAge = Math.clamp(V.potencyAge, 3, 18);
		options.addOption("Girls will not be able to impregnate others if their age is under", "potencyAge").showTextBox();

		options.addOption("Precocious puberty is", "precociousPuberty").addComment("Under certain conditions they can become pregnant or inseminate others younger then normal age - V.fertilityAge, though they may also experience delayed puberty.")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Age penalties are", "AgePenalty").addComment("Job and career penalties due to age.")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		options.addOption("Children growing as they age is", "loliGrow")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		el.append(options.render());

		return el;
	}

	function interfaceContent() {
		const el = new DocumentFragment();
		let options = new App.UI.OptionsGroup();

		options.addOption("Help tooltips are", "tooltipsEnabled")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off()
			.addComment(`This is mostly for new players. <span class="exampleTooltip.noteworthy">Colored text</span> can have tooltips.`);

		options.addOption("Accordion on week end defaults to", "useAccordion")
			.addValue("Open", 0).on().addValue("Collapsed", 1).off();

		options.addOption("Economic Tabs on weekly reports are", "useTabs")
			.addValue("Enabled", 1).on().addValue("Disabled", 0).off();

		el.append(options.render());

		App.UI.DOM.appendNewElement("h2", el, "Images");
		el.append(App.UI.artOptions());

		return el;
	}
};
