App.UI.Player = {};

App.UI.Player.appearance = function(options) {
	options.addOption("Your nationality is", "nationality", V.PC).showTextBox()
		.addValueList(App.Data.misc.baseNationalities)
		.addComment("For best result capitalize it.").pulldown();

	options.addOption("Your race is", "race", V.PC).showTextBox()
		.addValueList(Array.from(setup.filterRaces, (k => [k, k.toLowerCase()])));

	if (V.cheatMode) {
		options.addOption("Your race is", "origRace", V.PC).showTextBox()
			.addValueList(Array.from(setup.filterRaces, (k => [k, k.toLowerCase()])));
	}

	options.addOption("Your skin tone is", "skin", V.PC).showTextBox()
		.addValueList(makeAList(App.Medicine.Modification.naturalSkins));

	if (V.cheatMode) {
		options.addOption("Your genetic skin tone is", "origSkin", V.PC).showTextBox()
			.addValueList(makeAList(App.Medicine.Modification.naturalSkins));
	}

	options.addOption("Your body", "markings", V.PC)
		.addValueList([["Is clear of blemishes", "none"], ["Has light freckling", "freckles"], ["Has heavy freckling", "heavily freckled"]]);

	options.addOption("Your genetic eye color is", "origColor", V.PC.eye).showTextBox()
		.addValueList(makeAList(App.Medicine.Modification.eyeColor.map(color => color.value)));

	if (V.cheatMode) {
		options.addOption("Your original hair is", "origHColor", V.PC).showTextBox()
			.addValueList(makeAList(App.Medicine.Modification.Color.Primary.map(color => color.value)));
	} else {
		options.addOption("Your hair is", "hColor", V.PC).showTextBox()
			.addValueList(makeAList(App.Medicine.Modification.Color.Primary.map(color => color.value)));
	}

	function makeAList(iterable) {
		return Array.from(iterable, (k => [capFirstChar(k), k]));
	}
};

App.UI.Player.refreshmentChoice = function(options) {
	options.addOption("Your preferred refreshment is", "refreshment", V.PC).showTextBox()
		.addValue("Cigars", "cigar", () => { V.PC.refreshmentType = 0; })
		.addValue("Whiskey", "whiskey", () => { V.PC.refreshmentType = 1; });

	const option = options.addOption("Which you", "refreshmentType", V.PC)
		.addValueList(Array.from(App.Data.player.refreshmentType, (v, i) => [v, i]));

	let comment = `Flavor only; no mechanical effect. If entering a custom refreshment, please assign proper usage.`;
	if (V.PC.refreshmentType === 0) {
		comment += ` "Smoked" must fit into the following sentence: "I smoked a ${V.PC.refreshment}" to fit events properly.`;
	} else if (V.PC.refreshmentType === 5) {
		comment += ` "Popped" must fit into the following sentence: "I shook the bottle of ${V.PC.refreshment}" to fit events properly.`;
	} else if (V.PC.refreshmentType === 6) {
		comment += ` "Orally Dissolved" must fit into the following sentence: "I placed a tab of ${V.PC.refreshment} under my tongue" to fit events properly.`;
	}
	option.addComment(comment);
};

App.UI.Player.names = function(options) {
	options.addOption(`Everyone calls you <b>${PlayerName()}.</b>`);
	options.addOption("Your given name is", "slaveName", V.PC).showTextBox();

	if (V.cheatMode) {
		options.addOption("Birth Name", "birthName", V.PC).showTextBox();
	}

	if (V.PC.slaveSurname === 0) {
		options.addOption("And no surname", "slaveSurname", V.PC)
			.addValue("Add a surname", "Anon")
			.addComment("Surnames cannot be changed during the game outside of special circumstances.");
	} else {
		options.addOption("And your surname is", "slaveSurname", V.PC).showTextBox()
			.addValue("Go by a single name", 0)
			.addComment("Surnames cannot be changed during the game outside of special circumstances.");
		if (V.cheatMode) {
			options.addOption("Birth Surname", "birthSurname", V.PC).showTextBox();
		}
	}
};

App.UI.Player.design = function() {
	const el = new DocumentFragment();
	let options = new App.UI.OptionsGroup();
	let option;
	let r;
	let linkArray;
	const allowEdits = (V.freshPC === 1 || V.saveImported === 0 || V.cheatMode);

	// Title / age
	if (allowEdits) {
		options.addOption("You are a", "title", V.PC)
			.addValue("Masculine Master", 1, () => V.PC.genes = "XY").addValue("Feminine Mistress", 0, () => V.PC.genes = "XX");

		App.UI.Player.names(options);

		V.PC.physicalAge = V.PC.actualAge;
		V.PC.visualAge = V.PC.actualAge;
		if (V.cheatMode) {
			options.addOption("Actual Age", "actualAge", V.PC).showTextBox();
			options.addOption("Physical Age", "physicalAge", V.PC).showTextBox();
			options.addOption("Visual Age", "visualAge", V.PC).showTextBox();
			options.addOption("Ovary Age", "ovaryAge", V.PC).showTextBox();
			options.addOption("Age Implant", "ageImplant", V.PC).addValue("Age altering surgery", 1).on().addValue("No surgery", 0).off();
		} else {
			options.addOption("You are", "actualAge", V.PC).showTextBox()
				.addRange(25, 35, "<", "Surprisingly young").addRange(40, 50, "<", "Entering middle age")
				.addRange(55, 65, "<", "Well into middle age").addRange(70, 65, ">=", "Old");

			options.addOption(`Your birthday was <strong>${V.PC.birthWeek}</strong> weeks ago.`, "birthWeek", V.PC).showTextBox();
		}
	} else {
		r = [];
		r.push(`You are a`);
		if (V.PC.title === 1) {
			r.push(`masculine <strong>Master</strong>`);
		} else {
			r.push(`feminine ''Mistress'`);
		}
		r.push(`and everyone that matters calls you	${PlayerName()}.`);

		r.push(`You are ${V.PC.actualAge} years old which is`);
		if (V.PC.actualAge >= 65) {
			r.push(`<strong>old</strong>.`);
		} else if (V.PC.actualAge >= 50) {
			r.push(`<strong>well into middle age</strong>.`);
		} else if (V.PC.actualAge >= 35) {
			r.push(`<strong>entering middle age</strong>.`);
		} else {
			r.push(`<strong>surprisingly young</strong>.`);
		}
		App.Events.addNode(el, r, "p");
	}

	option = options.addOption("Player aging is", "playerAging")
		.addValue("Enabled", 2).on().addValue("Celebrate birthdays, but don't age.", 1).neutral().addValue("Disabled", 0).off();
	if (!V.cheatMode) {
		option.addComment("This option cannot be changed during the game.");
	}

	if (V.PC.customTitle) {
		options.addOption("Custom title", "customTitle", V.PC).showTextBox()
			.addValue("Reset to default", "", () => { delete V.PC.customTitle; delete V.PC.customTitleLisp; });

		options.addOption("Lisped custom title", "customTitleLisp", V.PC).showTextBox()
			.addComment('If using a custom title, select Master or Mistress to set the gender of your title. Make sure to replace your "s"s with "th"s to have working lisps in your lisped title.');
	} else {
		options.addOption("Custom title", "customTitle", V.PC)
			.addValue("Set custom title", "Master", () => V.PC.customTitleLisp = 'Mather');
	}

	// Appearance
	if (allowEdits) {
		App.UI.Player.appearance(options);

		options.addOption("Your face is", "faceShape", V.PC)
			.addValueList([
				["Normal", "normal"],
				["Androgynous", "androgynous"],
				["Masculine", "masculine"],
				["Cute", "cute"],
				["Sensual", "sensual"],
				["Exotic", "exotic"]
			]);
	} else {
		r = [];

		r.push(`You are a ${V.PC.nationality} ${V.PC.race} with`);
		if (V.PC.markings === "heavily freckled") {
			r.push(`heavily freckled`);
		} else if (V.PC.markings === "freckles") {
			r.push(`lightly freckled`);
		} else {
			r.push(`clear`);
		}
		r.push(`${V.PC.skin} skin, ${V.PC.hColor} hair and ${App.Desc.eyesColor(V.PC)}. You have a ${V.PC.faceShape} face.`);
		App.Events.addNode(el, r, "p");
	}

	// Refresh
	App.UI.Player.refreshmentChoice(options);

	// History
	if (allowEdits) {
		option = options.addOption("Before you came to the Free Cities, you were a", "career", V.PC);
		if (V.PC.career === "arcology owner") {
			option.addValue("Arcology owner", "arcology owner");
		} else {
			option.addValueList([
				["Member of the idle wealthy", "wealth"],
				["Business leader", "capitalist"],
				["Mercenary", "mercenary"],
				["Slaver", "slaver"],
				["Engineer", "engineer"],
				["Doctor", "medicine"],
				["Hacker", "BlackHat"],
				["Minor celebrity", "celebrity"],
				["Escort", "escort"],
				["Servant", "servant"],
				["Gang leader", "gang"]]
			);
			if (V.secExpEnabled > 0) {
				switch (V.PC.career) {
					case "capitalist":
						option.addComment(`<div><span class="yellowgreen">The propaganda hub's upgrades will be cheaper.</span></div>`);
						break;
					case "mercenary":
						option.addComment(`<div><span class="green">Easier to maintain security</span> and <span class="yellowgreen">the security HQ's upgrades will be cheaper.</span></div>`);
						break;
					case "slaver":
						option.addComment(`<div><span class="green">Easier to maintain authority</span> and <span class="yellowgreen">the security HQ's upgrades will be cheaper.</span></div>`);
						break;
					case "engineer":
						option.addComment(`<div><span class="yellowgreen">construction and upgrade of facilities will be cheaper.</span></div>`);
						break;
					case "medicine":
						option.addComment(`<div><span class="yellowgreen">Drug upgrades will be cheaper.</span></div>`);
						break;
					case "celebrity":
						option.addComment(`<div><span class="yellowgreen">The propaganda hub's upgrades will be cheaper.</span></div>`);
						break;
					case "escort":
						option.addComment(`<div><span class="red">Harder to maintain authority.</span></div>`);
						break;
					case "servant":
						option.addComment(`<div><span class="red">Harder to maintain authority.</span></div>`);
						break;
					case "gang":
						option.addComment(`<div><span class="green">Easier to maintain authority</span> and <span class="yellowgreen">the security HQ's upgrades will be cheaper.</span></div>`);
						break;
					case "BlackHat":
						option.addComment(`<div><span class="red">Harder to maintain authority.</span></div>`);
						break;
					default:
						option.addComment(`<div><span class="red">Harder to maintain authority,</span> but <span class="yellowgreen">the propaganda hub's upgrades will be cheaper.</span></div>`);
				}
			}
		}

		options.addOption("It is rumored that you acquired your arcology through", "rumor", V.PC)
			.addValueList([
				["Wealth", "wealth"],
				["Hard work", "diligence"],
				["Force", "force"],
				["Social engineering", "social engineering"],
				["Blind luck", "luck"]
			]);

		el.append(options.render());
	} else {
		r = [];
		switch (V.PC.career) {
			case "wealth":
				r.push(`Prior to being an arcology owner, you were a member of the idle wealthy.`);
				break;
			case "capitalist":
				r.push(`Prior to being an arcology owner, you were a business leader.`);
				break;
			case "mercenary":
				r.push(`Prior to being an arcology owner, you were a mercenary.`);
				break;
			case "slaver":
				r.push(`Prior to being an arcology owner, you were a slaver.`);
				break;
			case "engineer":
				r.push(`Prior to being an arcology owner, you were an engineer.`);
				break;
			case "medicine":
				r.push(`Prior to being an arcology owner, you were a surgeon.`);
				break;
			case "celebrity":
				r.push(`Prior to being an arcology owner, you were a minor celebrity.`);
				break;
			case "BlackHat":
				r.push(`Prior to being an arcology owner, you specialized in cracking databases and making mockeries of cyber security.`);
				break;
			case "arcology owner":
				r.push(`Being an arcology owner defines your life now.`);
				break;
			case "escort":
				r.push(`Prior to being an arcology owner, you knew how to survive off your looks and body.`);
				break;
			case "servant":
				r.push(`Prior to being an arcology owner, you served a well-off`);
				if (V.PC.counter.birthMaster >= 2) {
					r.push(`master and bore him several children.`);
				} else {
					r.push(`master.`);
				}
				break;
			case "gang":
				r.push(`Prior to being an arcology owner, you were the leader of a ruthless gang.`);
				break;
		}

		r.push(`Word in the arcology is you acquired it through`);
		switch (V.PC.rumor) {
			case "wealth":
				r.push(`a rather ridiculous amount of money.`);
				break;
			case "diligence":
				r.push(`sheer effort.`);
				break;
			case "force":
				r.push(`brutal force.`);
				break;
			case "social engineering":
				r.push(`clever social manipulation.`);
				break;
			case "luck":
				r.push(`blind luck.`);
				break;
		}
		App.Events.addNode(el, r, "p");
	}

	// Sexuality
	App.UI.DOM.appendNewElement("h2", el, "Sexuality");

	if (allowEdits) {
		options = new App.UI.OptionsGroup();

		if (V.PC.vagina !== -1 && V.PC.dick !== 0) {
			State.temporary.vaginaPenis = 2;
		} else if (V.PC.vagina !== -1) {
			State.temporary.vaginaPenis = 1;
		} else {
			State.temporary.vaginaPenis = 0;
		}

		option = options.addOption("You have a", "vaginaPenis", State.temporary)
			.addValue("Penis", 0, () => {
				V.PC.preg = 0;
				V.PC.pregType = 0;
				V.PC.dick = 4;
				V.PC.balls = 3;
				V.PC.scrotum = 3;
				V.PC.prostate = 1;
				V.PC.vagina = -1;
				V.PC.ovaries = 0;
			}).addValue("Vagina", 1, () => {
				V.PC.dick = 0;
				V.PC.balls = 0;
				V.PC.scrotum = 0;
				V.PC.prostate = 0;
				V.PC.vagina = 1;
				V.PC.ovaries = 1;
			}).addValue("Penis and Vagina", 2, () => {
				V.PC.dick = 4;
				V.PC.balls = 3;
				V.PC.scrotum = 3;
				V.PC.prostate = 1;
				V.PC.vagina = 1;
				V.PC.ovaries = 1;
			});
		if (State.temporary.vaginaPenis === 0) {
			option.addComment("Standard sex scenes; easiest reputation maintenance.");
		} else if (State.temporary.vaginaPenis === 1) {
			option.addComment("Sex scene variations; most difficult reputation maintenance.");
		} else {
			option.addComment("Sex scene variations; more difficult reputation maintenance; some unique opportunities, especially with breasts.");
		}

		if (V.cheatMode) {
			options.addOption("Vagina", "vagina", V.PC).showTextBox();
			options.addOption("New vagina", "newVag", V.PC).showTextBox();
			options.addOption("Dick", "dick", V.PC).showTextBox();
			options.addOption("Balls", "balls", V.PC).addValueList([
				["Normal", 3],
				["Big", 5],
				["Huge", 9],
				["Monstrous", 30]
			])
				.showTextBox();
			options.addOption("Balls implant", "ballsImplant", V.PC).showTextBox();
		}

		if (V.PC.vagina !== -1) {
			option = options.addOption("You are", "preg", V.PC)
				.addValue("Taking contraceptives", -1, () => { V.PC.pregType = 0; V.PC.labor = 0; })
				.addValue("Not taking contraceptives", 0, () => { V.PC.pregType = 0; V.PC.labor = 0; })
				.addRange(16, 37, "<=", "Pregnant").addCallback(() => { V.PC.pregType = 1; V.PC.labor = 0; })
				.addRange(40, 42, "<=", "Ready to drop").addCallback(() => { V.PC.pregType = 1; V.PC.labor = 0; })
				.addRange(43, 42, ">", "Ready to drop with octuplets").addCallback(() => { V.PC.pregType = 8; V.PC.labor = 1; });
			const r =[];
			if (V.cheatMode) {
				option.showTextBox();
				r.push(`How far along your pregnancy is (pregMood kicks in at 24+ weeks) - -2: infertile, -1: contraceptives, 0: not pregnant, 1 - 42: pregnant, 43+: giving birth.`);
			}
			if (V.PC.preg === -1) {
				r.push("You can't get pregnant, however there will be a slight increase to living expenses.");
			}

			if (V.PC.counter.birthsTotal > 0) {
				r.push(`You have given birth to <strong>${V.PC.counter.birthsTotal}</strong> babies.`);
			}
			if (r.length >0) {
				option.addComment(r.join(" "));
			}

			option = options.addOption("Hormone effects", "pregMood", V.PC)
				.addValueList([
					["None", 0],
					["Caring and motherly", 1],
					["Aggressive and domineering", 2]
				]);
			if (V.PC.pregMood === 1) {
				option.addComment("Sex scene alterations; slaves will trust you more, but may try to take advantage of your mercy.");
			} else if (V.PC.pregMood === 2) {
				option.addComment("Sex scene alterations; slaves will fear you more, but will become more submissive to you.");
			}

			if (V.cheatMode) {
				options.addOption("Fetus Count", "pregType", V.PC).showTextBox().addComment(`how many you're having (1-8)`);
				options.addOption("Pregnancy Source", "pregSource", V.PC)
					.addValueList([
						["Unknown", 0],
						["Self-impregnation", -1],
						["Citizen", -2],
						["Former Master", -3],
						["Male arc owner", -4],
						["Client", -5],
						["Societal Elite", -6],
						["Designer baby", -7],
						["Futanari Sister", -9],
					])
					.showTextBox();
			}
		}

		if (V.PC.title === 1 && V.PC.boobs <= 100) {
			option = options.addOption("Your chest is", "boobs", V.PC).addValue("Manly", 100, () => V.PC.boobsImplant = 0);
		} else {
			option = options.addOption("Your breasts are", "boobs", V.PC).addValue("Flat", 100, () => V.PC.boobsImplant = 0);
		}
		option.addValueList([
			["C-cups", 500],
			["DD-cups", 900],
			["F-cups", 1100],
			["G-cups", 1300]
		]);
		option.showTextBox("CCs");

		if (V.PC.boobs >= 500) {
			options.addOption("Your breasts are", "boobsImplant", V.PC)
				.addValueList([
					["All natural", 0],
					["Fake", 400]
				])
				.showTextBox("CCs");
		}

		if (V.cheatMode) {
			if (V.PC.boobs >= 500) {
				options.addOption("Your breasts are", "lactation", V.PC)
					.addValueList([
						["Lactating", 1],
						["Not Lactating", 0]
					]);
			}
		}

		options.addOption("Your butt size", "butt", V.PC)
			.addValueList([
				["Normal", 2],
				["Big", 3],
				["Huge", 4],
				["Enormous", 5],
			])
			.showTextBox();

		options.addOption("Your butt is", "buttImplant", V.PC)
			.addValueList([
				["All natural", 0],
				["Fake", 1]
			])
			.showTextBox("CCs");


		options.addOption("You are genetically", "genes", V.PC)
			.addValue("XY").addValue("XX");

		el.append(options.render());
	} else {
		r = [];
		r.push(`You have a`);
		if (V.PC.vagina !== -1 && V.PC.dick !== 0) {
			r.push(`penis and vagina`);
		} else if (V.PC.dick !== 0) {
			r.push(`penis.`);
		} else if (V.PC.vagina !== -1) {
			r.push(`vagina`);
		}
		if (V.PC.vagina !== -1) {
			r.push(`and are`);
			if (V.PC.pregWeek < 0) {
				r.push(`recovering from your last pregnancy.`);
			} else if (V.PC.preg === -2) {
				r.push(`infertile.`);
			} else if (V.PC.preg === -1) {
				r.push(`taking contraceptives.`);
			} else if (V.PC.preg === 0) {
				r.push(`fertile.`);
			} else if (V.PC.preg > 37) {
				r.push(`extremely pregnant.`);
			} else if (V.PC.preg > 0) {
				r.push(`pregnant.`);
			}

			linkArray = [];
			if (V.PC.preg > 20 || V.PC.counter.birthsTotal > 0) {
				if (V.PC.pregMood === 1) {
					r.push(`You tend to be caring and motherly when you're pregnant.`);
					linkArray.push(noChange(), aggressive());
				} else if (V.PC.pregMood === 0) {
					r.push(`Pregnancy doesn't really affect your mood.`);
					linkArray.push(motherly(), aggressive());
				} else {
					r.push(`You tend to be very demanding and aggressive when you're pregnant.`);
					linkArray.push(noChange(), motherly());
				}
			} else {
				if (V.PC.pregMood === 1) {
					r.push(`You tend to be caring and motherly when you're hormonal.`);
					linkArray.push(noChange(), aggressive());
				} else if (V.PC.pregMood === 0) {
					r.push(`Your mood isn't tied to your hormones.`);
					linkArray.push(motherly(), aggressive());
				} else {
					r.push(`You tend to be very demanding and aggressive when you're hormonal.`);
					linkArray.push(noChange(), motherly());
				}
			}

			r.push(App.UI.DOM.generateLinksStrip(linkArray));
			if (V.PC.counter.birthsTotal > 0) {
				r.push(`You have given birth to ${V.PC.counter.birthsTotal} babies.`);
			}
		}

		if (V.PC.boobs >= 300) {
			r.push(`You have a`);
			if (V.PC.title > 0) {
				r.push(`masculine`);
			} else {
				r.push(`feminine`);
			}
			r.push(`body with`);
			if (V.PC.boobs >= 1400) {
				r.push(`giant${(V.PC.boobsImplant !== 0) ? `, fake` : ``} cow tits.`);
			} else if (V.PC.boobs >= 1200) {
				r.push(`huge`);
				if (V.PC.boobsImplant !== 0) {
					r.push(`fake`);
				}
				r.push(`breasts.`);
			} else if (V.PC.boobs >= 1000) {
				r.push(`big`);
				if (V.PC.boobsImplant !== 0) {
					r.push(`fake`);
				}
				r.push(`breasts.`);
			} else if (V.PC.boobs >= 800) {
				r.push(`noticeable breasts.`);
			} else if (V.PC.boobs >= 650) {
				r.push(`unremarkable breasts.`);
			} else if (V.PC.boobs >= 500) {
				r.push(`average breasts.`);
			} else {
				r.push(`small breasts.`);
			}
		} else {
			if (V.PC.title > 0) {
				r.push(`You have a manly chest.`);
			} else {
				r.push(`You are flat as a board.`);
			}
		}
		App.Events.addNode(el, r, "p");
	}


	if (V.cheatMode) {
		// Skills
		App.UI.DOM.appendNewElement("h2", el, "Skills");
		options = new App.UI.OptionsGroup();
		options.addOption(`Trading: ${tradingDescription()}`, "trading", V.PC.skill).addValueList([
			["Economics master", 100],
			["Economics expert", 90],
			["Skilled in economics", 70],
			["Amateur economist", 50],
			["Economics beginner", 30],
			["Basic trader", 0],
			["Haggler", -10],
			["Shopper", -30],
			["Weak saver", -50],
			["Money sieve", -70],
			["What's a trading?", -90],
		]).showTextBox();

		options.addOption(`Warfare: ${warfareDescription()}`, "warfare", V.PC.skill).addValueList([
			["Warfare master", 100],
			["Warfare expert", 90],
			["Skilled in warfare", 70],
			["Amateur combatant", 50],
			["Combat beginner", 30],
			["Basic fighter", 0],
			["Gun haver", -10],
			["Knife holder", -30],
			["Throat puncher", -50],
			["Groin kicker?", -70],
			["Most likely to be raped", -90]
		]).showTextBox();

		options.addOption(`Slaving: ${slavingDescription()}`, "slaving", V.PC.skill).addValueList([
			["Master slaver", 100],
			["Expert slaver", 90],
			["Skilled in slaving", 70],
			["Amateur slaver", 50],
			["Slaving beginner", 30],
			["Basic slaver", 0],
			["Can't make me a slave", -10],
			["Can read contracts", -30],
			["Careful now", -50],
			["Don't trust that guy", -70],
			["Potential slave", -90]
		]).showTextBox();

		options.addOption(`Engineering: ${engineeringDescription()}`, "engineering", V.PC.skill).addValueList([
			["Master engineer", 100],
			["Expert engineer", 90],
			["Skilled in engineering", 70],
			["Amateur engineer", 50],
			["Engineering beginner", 30],
			["Basic engineer", 0],
			["Gingerbread house", -10],
			["Knot tyer", -30],
			["You can use glue", -50],
			["You aren't handy", -70],
			["My hovercraft is full of eels", -90]
		]).showTextBox();

		options.addOption(`Medicine: ${medicineDescription()}`, "medicine", V.PC.skill).addValueList([
			["Master surgeon", 100],
			["Expert surgeon", 90],
			["Skilled in medicine", 70],
			["Amateur surgeon", 50],
			["Medical beginner", 30],
			["Basic medic", 0],
			["Can treat wounds", -10],
			["First-aid kit user", -30],
			["Band-aid applier", -50],
			["MEDIC!", -70],
			["Give me another beer", -90]
		]).showTextBox();

		options.addOption(`Hacking: ${hackingDescription()}`, "hacking", V.PC.skill).addValueList([
			["Master hacker", 100],
			["Expert hacker", 90],
			["Skilled hacker", 70],
			["Amateur hacker", 50],
			["Hacking beginner", 30],
			["Basic hacker", 0],
			["Mouse clicker", -10],
			["You can press Enter", -30],
			[`Where's the "any" key?`, -50],
			["Main screen turn on?", -70],
			["Ooh, cool glowy thingy!", -90]
		]).showTextBox();

		el.append(options.render());

		// Family
		App.UI.DOM.appendNewElement("h2", el, "Family");
		options = new App.UI.OptionsGroup();
		options.addOption(`Your mother ID`, "mother", V.PC).showTextBox();
		options.addOption(`Your father ID`, "father", V.PC).showTextBox();
		el.append(options.render());

		// Potential
		App.UI.DOM.appendNewElement("h2", el, "Misc");
		options = new App.UI.OptionsGroup();
		options.addOption(`Sexual Energy`, "sexualEnergy", V.PC).showTextBox();
		options.addOption(`Cum Tap`, "cumTap", V.PC.skill).showTextBox();
		options.addOption(`Stored Cum`, "storedCum", V.PC.counter).showTextBox();
		options.addOption(`Fertility Drugs`, "fertDrugs", V.PC)
			.addValue("Yes", 1).on()
			.addValue("No", 0).off();
		options.addOption(`Forced Fertility Drugs`, "forcedFertDrugs", V.PC).showTextBox();
		options.addOption(`Stamina Pills`, "staminaPills", V.PC)
			.addValue("Yes", 1).on()
			.addValue("No", 0).off();
		el.append(options.render());
	}

	return el;

	function noChange() {
		return App.UI.DOM.link(
			"Change to no change",
			() => { V.PC.pregMood = 0; },
			[],
			"Intro Summary"
		);
	}

	function motherly() {
		return App.UI.DOM.link(
			"Change to motherly",
			() => { V.PC.pregMood = 1; },
			[],
			"Intro Summary"
		);
	}

	function aggressive() {
		return App.UI.DOM.link(
			"Change to aggressive",
			() => { V.PC.pregMood = 2; },
			[],
			"Intro Summary"
		);
	}

	function tradingDescription() {
		if (V.PC.skill.trading >= 100) {
			return `You are a master at economics and trading.`;
		} else if (V.PC.skill.trading >= 80) {
			return `You are an expert at economics and trading.`;
		} else if (V.PC.skill.trading >= 60) {
			return `You are skilled in economics and trading.`;
		} else if (V.PC.skill.trading >= 40) {
			return `You know some things about economics and trading.`;
		} else if (V.PC.skill.trading >= 20) {
			return `You are a beginner in economics.`;
		} else if (V.PC.skill.trading >= 0) {
			return `You know only the basics of trading.`;
		} else if (V.PC.skill.trading >= -20) {
			return `You know how to haggle a little.`;
		} else if (V.PC.skill.trading >= -40) {
			return `You know how to shop around.`;
		} else if (V.PC.skill.trading >= -60) {
			return `You know not to pay sticker price.`;
		} else if (V.PC.skill.trading >= -80) {
			return 	`People always give you discounts, but you never save any money.`;
		} else {
			return `They said it was a bear market, so where are the bears?`;
		}
	}

	function warfareDescription() {
		if (V.PC.skill.warfare >= 100) {
			return `You are a master of warfare.`;
		} else if (V.PC.skill.warfare >= 80) {
			return `You are an expert at tactics and strategy.`;
		} else if (V.PC.skill.warfare >= 60) {
			return `You are skilled in combat.`;
		} else if (V.PC.skill.warfare >= 40) {
			return `You know some things about combat.`;
		} else if (V.PC.skill.warfare >= 20) {
			return `You are a beginner in tactics and strategy.`;
		} else if (V.PC.skill.warfare >= 0) {
			return `You know only the basics of fighting.`;
		} else if (V.PC.skill.warfare >= -20) {
			return `You know how to hold a gun.`;
		} else if (V.PC.skill.warfare >= -40) {
			return `You know how to stab with a knife.`;
		} else if (V.PC.skill.warfare >= -60) {
			return `Go for the throat?`;
		} else if (V.PC.skill.warfare >= -80) {
			return `Just kick them in the balls, right?`;
		} else {
			return `People like you are usually the first raped in a war.`;
		}
	}

	function slavingDescription() {
		if (V.PC.skill.slaving >= 100) {
			return `You are a master slaver.`;
		} else if (V.PC.skill.slaving >= 80) {
			return `You are an expert at enslaving.`;
		} else if (V.PC.skill.slaving >= 60) {
			return `You are skilled in slaving.`;
		} else if (V.PC.skill.slaving >= 40) {
			return `You know some things about getting slaves.`;
		} else if (V.PC.skill.slaving >= 20) {
			return `You are a beginner in slaving.`;
		} else if (V.PC.skill.slaving >= 0) {
			return `You know only the basics of slaving.`;
		} else if (V.PC.skill.slaving >= -20) {
			return `You know how to avoid becoming a slave.`;
		} else if (V.PC.skill.slaving >= -40) {
			return `You know to read contracts before you sign them.`;
		} else if (V.PC.skill.slaving >= -60) {
			return `You know to be careful.`;
		} else if (V.PC.skill.slaving >= -80) {
			return `You know better than to trust anyone.`;
		} else {
			return `It would be easy to enslave you.`;
		}
	}

	function engineeringDescription() {
		if (V.PC.skill.engineering >= 100) {
			return `You are a master engineer.`;
		} else if (V.PC.skill.engineering >= 80) {
			return `You are an expert at engineering.`;
		} else if (V.PC.skill.engineering >= 60) {
			return `You are skilled in engineering.`;
		} else if (V.PC.skill.engineering >= 40) {
			return `You know some things about engineering.`;
		} else if (V.PC.skill.engineering >= 20) {
			return `You are a beginner in engineering.`;
		} else if (V.PC.skill.engineering >= 0) {
			return `You know only the basics of engineering.`;
		} else if (V.PC.skill.engineering >= -20) {
			return `You can build a gingerbread house that doesn't collapse.`;
		} else if (V.PC.skill.engineering >= -40) {
			return `You can tie a tight knot, does that count?`;
		} else if (V.PC.skill.engineering >= -60) {
			return `Glue is your friend; lots of it.`;
		} else if (V.PC.skill.engineering >= -80) {
			return `You know better than to even try to build something.`;
		} else {
			return `You can cook; that's sort of like building something, right?`;
		}
	}

	function medicineDescription() {
		if (V.PC.skill.medicine >= 100) {
			return `You are a master surgeon.`;
		} else if (V.PC.skill.medicine >= 80) {
			return `You are an expert at medicine and surgery.`;
		} else if (V.PC.skill.medicine >= 60) {
			return `You are skilled in surgery.`;
		} else if (V.PC.skill.medicine >= 40) {
			return `You know some things about medicine.`;
		} else if (V.PC.skill.medicine >= 20) {
			return `You are a beginner in medicine.`;
		} else if (V.PC.skill.medicine >= 0) {
			return `You know the basics of treating injuries.`;
		} else if (V.PC.skill.medicine >= -20) {
			return `You can stop a wound from getting infected.`;
		} else if (V.PC.skill.medicine >= -40) {
			return `Gauze is your friend. Just keep wrapping.`;
		} else if (V.PC.skill.medicine >= -60) {
			return `You know how to apply a band-aid.`;
		} else if (V.PC.skill.medicine >= -80) {
			return `Cure-alls are wonderful. Why aren't they sold in stores, though?`;
		} else {
			return `Alcohol makes pain go away, right?`;
		}
	}

	function hackingDescription() {
		if (V.PC.skill.hacking >= 100) {
			return `You are a master of hacking.`;
		} else if (V.PC.skill.hacking >= 80) {
			return `You are an expert at hacking.`;
		} else if (V.PC.skill.hacking >= 60) {
			return `You are skilled in hacking.`;
		} else if (V.PC.skill.hacking >= 40) {
			return `You know some things about hacking.`;
		} else if (V.PC.skill.hacking >= 20) {
			return `You are a beginner in hacking.`;
		} else if (V.PC.skill.hacking >= 0) {
			return `You know only the basics of hacking.`;
		} else if (V.PC.skill.hacking >= -20) {
			return `You know how to click a mouse.`;
		} else if (V.PC.skill.hacking >= -40) {
			return `Enter does something?`;
		} else if (V.PC.skill.hacking >= -60) {
			return `Where is the "any" key?`;
		} else if (V.PC.skill.hacking >= -80) {
			return `You can push the power button, good job.`;
		} else {
			return `This black box thingy is magical.`;
		}
	}
};
