/**
 * UI for the Salon.  Refreshes without refreshing the passage.
 * @param {App.Entity.SlaveState} slave
 * @param {boolean} cheat if true, will hide scenes and keep the player from being billed for changes.
 */
App.UI.salon = function(slave, cheat = false) {
	const container = document.createElement("span");
	container.id = "salon";
	const {
		He, His,
		he, his, him
	} = getPronouns(slave);
	Enunciate(slave);

	container.append(createPage());
	return container;

	function createPage() {
		const el = new DocumentFragment();
		if (!cheat) {
			if (V.seeImages > 0) {
				App.Events.drawEventArt(el, slave);
			}
			el.append(intro());
		}
		el.append(eyewear());
		el.append(ears());
		if (slave.horn !== "none") {
			el.append(horns());
		}
		el.append(hair());
		el.append(makeup());
		el.append(nails());
		el.append(skin());
		el.append(bodyHair());
		return el;
	}

	function intro() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h1", el, "The Auto Salon");
		App.UI.DOM.appendNewElement("div", el, `${SlaveFullName(slave)} is seated in the auto salon. ${He} is awaiting your artistic pleasure.`, "scene-intro");
		return el;
	}

	function eyewear() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h3", el, "Eyewear");
		const r = [];
		const options = new App.UI.OptionsGroup();

		if (getBestVision(slave) === 0) {
			r.push(`${He} is blind`);
		} else if (anyVisionEquals(slave, 1)) {
			r.push(`${He} is nearsighted`);
		} else {
			r.push(`${His} vision is normal`);
		}

		const option = options.addOption(r.join(" "), "eyewear", slave)
			.addValue("None", "none")
			.addValue("Cosmetic glasses", "glasses", billMod);
		if (getBestVision(slave) !== 0 && anyVisionEquals(slave, 1)) {
			option.addValue("Corrective glasses", "corrective glasses", billMod);
			option.addValue("Corrective glasses", "corrective glasses", billMod);
		} else {
			option.addValue("Blurring glasses", "blurring glasses", billMod);
			option.addValue("Blurring glasses", "blurring glasses", billMod);
			option.addComment("Blurring options are annoying and impede performance on some assignments.");
		}
		el.append(options.render());

		el.append(App.Medicine.Modification.eyeSelector(slave, cheat));
		return el;
	}

	function ears() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h3", el, "Ears");
		const r = [];
		const options = new App.UI.OptionsGroup();

		if (slave.earImplant === 1) {
			r.push(`${He} has artificial inner ear implants`);
		} else if (slave.hears < -1) {
			r.push(`${He} is deaf`);
		} else if (slave.hears > -1) {
			r.push(`${His} hearing is normal`);
		} else {
			r.push(`${He} is hearing impaired`);
		}
		const option = options.addOption(r.join(" "), "earwear", slave)
			.addValue("None", "none");
		// Hard of hearing
		if (slave.hears === -1 && slave.earImplant !== 1) {
			option.addValue("Hearing aids", "hearing aids", billMod);
		}
		// Is not deaf
		if (slave.hears > -2 || slave.earImplant === 1) {
			option.addValue("Muffling ear plugs", "muffling ear plugs", billMod);
			option.addValue("Deafening ear plugs", "deafening ear plugs", billMod);
		}
		el.append(options.render());

		el.append(App.Medicine.Salon.ears(slave));

		return el;
	}

	function horns() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h3", el, "Horns");
		const options = new App.UI.OptionsGroup();

		const option = options.addOption(`Set the color of ${his} ${slave.horn}`, "hornColor", slave);
		for (const hornColor of App.Medicine.Modification.hornColor) {
			option.addValue(capFirstChar(hornColor), hornColor, billMod);
		}

		el.append(options.render());

		return el;
	}

	function makeup() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h3", el, "Makeup");
		const options = new App.UI.OptionsGroup();

		options.addOption(App.Desc.makeup(slave), "makeup", slave)
			.addValue("Makeup free", 0)
			.addValue("Nice", 1, billMod)
			.addValue("Gorgeous", 2, billMod)
			.addValue("Slutty", 4, billMod)
			.addValue("Color-coordinate with hair", 3, billMod);

		options.addOption("", "makeup", slave)
			.addValue("Neon", 5, billMod)
			.addValue("Neon, color-coordinate with hair", 6, billMod);

		options.addOption("", "makeup", slave)
			.addValue("Metallic", 7, billMod)
			.addValue("Metallic, color-coordinate with hair", 8, billMod);

		el.append(options.render());

		return el;
	}

	function nails() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h3", el, "Nails");
		const options = new App.UI.OptionsGroup();

		options.addOption(App.Desc.nails(slave), "nails", slave)
			.addValue("Neatly clipped", 0, billMod)
			.addValue("Long and elegant", 1, billMod)
			.addValue("Sharp and claw-like", 3, billMod)
			.addValue("Bright and glittery", 4, billMod)
			.addValue("Very long and garish", 4, billMod)
			.addValue("Color-coordinate with hair", 2, billMod);

		options.addOption("", "nails", slave)
			.addValue("Neon", 6, billMod)
			.addValue("Neon, color-coordinate with hair", 7, billMod);

		options.addOption("", "nails", slave)
			.addValue("Metallic", 8, billMod)
			.addValue("Metallic, color-coordinate with hair", 9, billMod);

		el.append(options.render());

		return el;
	}

	function hair() {
		const el = new DocumentFragment();
		let option;
		App.UI.DOM.appendNewElement("h3", el, "Hair");
		const options = new App.UI.OptionsGroup();
		let title;
		let showChoices = true;
		const hasWig = (slave.bald === 1 && slave.hStyle !== "bald");

		if (slave.bald === 1) {
			if (slave.hStyle === "bald") {
				title = `${He} is completely bald.`;
				showChoices = false;
			} else {
				title = `${His} wig is ${slave.hColor}.`;
			}
		} else {
			title = `${His} hair is ${slave.hColor}.`;
		}
		App.UI.DOM.appendNewElement("div", el, title);

		if (slave.bald === 1) {
			options.addOption(`Use a wig`, "hStyle", slave)
				.addValue("Enable", "neat").on()
				.addValue("Disable", "bald").off();
		}

		if (showChoices) {
			if (slave.hLength > 1) {
				// Original color
				if (cheat) {
					option = options.addOption("Natural color", "origHColor", slave);
					for (const color of App.Medicine.Modification.Color.Primary) {
						option.addValue(capFirstChar(color.value), color.value, billMod);
					}
					option.pulldown();

					options.addOption("")
						.customButton(
							"Sync body hair color",
							() => {
								slave.eyebrowHColor = slave.origHColor;
								slave.pubicHColor = slave.origHColor;
								slave.underArmHColor = slave.origHColor;
								App.UI.reload();
							},
							""
						);
				}
				// Color
				option = options.addOption("Primary color", "hColor", slave);
				if (slave.origHColor !== slave.hColor) {
					option.addValue("Restore natural color", slave.origHColor, billMod);
				}
				for (const color of App.Medicine.Modification.Color.Primary) {
					option.addValue(capFirstChar(color.value), color.value, billMod);
				}
				option.pulldown();

				option = options.addOption("Secondary color", "hColor", slave);
				for (const color of App.Medicine.Modification.Color.Secondary) {
					option.addValue(color.title, (slave.hColor + color.value), billMod);
				}
				option.pulldown();
			}
			// Style
			if (slave.hLength > 1) {
				title = `Style ${hasWig ? "wig" : "hair"} `;
			} else {
				title = `${His} ${hasWig ? "wig" : "hair"} is too short to style meaningfully`;
			}
			option = options.addOption(title, "hStyle", slave);
			if (slave.hLength > 1) {
				for (const style of App.Medicine.Modification.hairStyles.Normal) {
					option.addValue(style.title, style.value, billMod);
				}
				option.pulldown();
			}

			// Style + Cut
			if (slave.hLength > 1) {
				option = options.addOption(`${hasWig ? "Change wig style and length" : "Cut and style hair"}`, "hStyle", slave);
				if (slave.hLength > 1) {
					for (const style of App.Medicine.Modification.hairStyles.Cut) {
						option.addValue(
							style.title,
							style.value,
							() => {
								slave.hLength = style.hLength;
								billMod();
							});
					}
				}
			}

			// Length
			option = options.addOption(`${hasWig ? "Find longer or shorter wig" : "Cut or lengthen hair"}`, "hLength", slave);
			if (slave.hLength > 0) {
				for (const style of App.Medicine.Modification.hairStyles.Length) {
					if (
						(style.hasOwnProperty("requirements") && !style.requirements(slave)) ||
						(style.hLength && style.hLength > slave.hLength)
					) {
						continue;
					}
					option.addValue(style.title, style.hLength, billMod);
				}
				if (!slave.bald && slave.hLength < 150) {
					option.addValue("Apply extensions", slave.hLength + 10, billMod);
				}
			} else {
				option.addValue("Apply hair growth stimulating treatment", 1);
			}

			option.showTextBox();

			// Maintain
			if (!hasWig) {
				options.addOption(`Maintain this length`, "haircuts", slave)
					.addValue("Enable", 1).on()
					.addValue("Disable", 0).off();
			}
		}

		el.append(options.render());
		return el;
	}

	function skin() {
		const el = new DocumentFragment();
		let r;
		let option;
		App.UI.DOM.appendNewElement("h3", el, "Skin");
		const options = new App.UI.OptionsGroup();
		let comment = [];

		if (cheat) {
			option = options.addOption(`${His} natural skin color is`, "origSkin", slave).showTextBox().pulldown();
			for (const skin of App.Medicine.Modification.naturalSkins) {
				option.addValue(capFirstChar(skin), skin, () => slave.skin = slave.origSkin);
			}
		}

		option = options.addOption(`${His} skin is ${slave.skin}.`, "skin", slave);
		if (App.Medicine.Modification.dyedSkins.includes(slave.skin)) {
			option.addValue("Remove coloring", slave.origSkin, billMod);
		} else if ((slave.skin === "sun tanned") || (slave.skin === "spray tanned")) {
			option.addValue("Remove tanning", slave.origSkin, billMod);
		}

		if (!App.Medicine.Modification.dyedSkins.includes(slave.skin)) {
			if (slave.skin === "sun tanned" || slave.skin === "spray tanned") {
				comment.push(`${His} skin tanning must be removed before any advanced procedure to change ${his} skin color.`);
			} else {
				if (skinToneLevel(slave.skin) > 1) {
					option.addValue("Bleach", changeSkinTone(slave.skin, -2), billMod);
				}
				if (skinToneLevel(slave.skin) > 8) {
					option.addValue("Lighten", changeSkinTone(slave.skin, -1), billMod);
				}
				if (skinToneLevel(slave.skin) < 18) {
					option.addValue("Darken", changeSkinTone(slave.skin, 1), billMod);
				}
				if (skinToneLevel(slave.skin) < 25) {
					option.addValue("Blacken", changeSkinTone(slave.skin, 2), billMod);
				}
			}
			if (slave.skin !== "sun tanned") {
				if (skinToneLevel(slave.skin) < 6) {
					comment.push(`${His} skin is so light in color that any attempt at natural tanning is more likely to damage ${his} skin.`);
				} else if ((skinToneLevel(slave.skin) > 20)) {
					comment.push(`${His} skin is so dark in color that any attempt at natural tanning is not likely to appear on ${his} skin.`);
				} else {
					option.addValue("Sun tan", "sun tanned", billMod);
				}
			}
			if (slave.skin !== "spray tanned") {
				option.addValue("Spray tan", "sun tanned", billMod);
			}
			option.addComment(comment.join(" "));
		}

		option = options.addOption(`Dye or paint`, "skin", slave);
		for (const dye of App.Medicine.Modification.dyedSkins) {
			option.addValue(capFirstChar(dye), dye, billMod);
		}

		if (cheat) {
			options.addOption(`${His} skin's marks are`, "markings", slave)
				.addValueList([
					["None", "none"],
					["Freckles", "freckles"],
					["Heavily freckled", "heavily freckled"],
					["Beauty mark", "beauty mark"],
					["Birthmark", "birthmark"],
				]);
		}
		option.pulldown();

		if (slave.markings === "beauty mark") {
			r = [];
			r.push(`${He} has a prominent mole on ${his} face, which`);
			if (slave.face > 40) {
				r.push(`qualifies as a beauty mark and enhances ${his} attractiveness due to ${his} facial beauty.`);
			} else if (slave.face < -10) {
				r.push(`makes ${him} even less attractive.`);
			} else {
				r.push(`qualifies as a beauty mark since ${he}'s pretty, having no significant impact on ${his} beauty.`);
			}
			options.addOption(r.join(" "), "markings", slave)
				.addValue("Remove it", "none", billMod);
		}
		if (slave.markings === "birthmark") {
			r = [];
			r.push(`${He} has a large birthmark, which`);
			if (slave.prestige > 0 || slave.porn.prestige > 1) {
				r.push(`enhances ${his} attractiveness due to ${his} prestige.`);
			} else {
				r.push(`detracts from ${his} attractiveness.`);
			}
			options.addOption(r.join(" "), "markings", slave)
				.addValue("Bleach it", "none", billMod);
		}

		el.append(options.render());

		return el;
	}

	function bodyHair() {
		const el = new DocumentFragment();
		const options = new App.UI.OptionsGroup();
		let option;
		let r = [];
		App.UI.DOM.appendNewElement("h3", el, "Body hair");

		// Eyebrows
		if (slave.eyebrowHStyle !== "bald") {
			// Describe and change color
			r.push(`${His} eyebrows`);
			if (slave.eyebrowHStyle === "shaved") {
				r.push(`would be ${slave.eyebrowHColor} if present.`);
			} else {
				r.push(`are ${slave.eyebrowHColor}.`);
			}

			option = options.addOption(r.join(" "), "eyebrowHColor", slave);
			if (slave.eyebrowHColor !== slave.hColor) {
				option.addValue("Match the hair", slave.hColor);
			}
			option.addValueList(makeAList(App.Medicine.Modification.Color.Primary.map(color => color.value)));
			option.addCallbackToEach(billMod);
			option.pulldown();

			// Style
			option = options.addOption(`Style ${his} eyebrow hair`, "eyebrowHStyle", slave);
			for (const fullness of App.Medicine.Modification.eyebrowStyles) {
				option.addValue(capFirstChar(fullness), fullness, billMod);
			}
			option.pulldown();

			// Fullness
			option = options.addOption(`Shape ${his} eyebrow hair`, "eyebrowFullness", slave);
			for (const fullness of App.Medicine.Modification.eyebrowFullness) {
				option.addValue(capFirstChar(fullness), fullness, billMod);
			}
			option.pulldown();
		} else {
			options.addComment(`${His} eyebrows are completely hairless.`);
		}

		// Pubic hair
		const _pubertyAge = Math.min(slave.pubertyAgeXX, slave.pubertyAgeXY);
		r = [];
		const hasPubes = (slave.pubicHStyle !== "bald" && slave.pubicHStyle !== "hairless" && slave.physicalAge >= _pubertyAge - 1);
		if (hasPubes) {
			r.push(`${His}`);
			if (slave.physicalAge < _pubertyAge) {
				r.push(`wispy pubic hair, which is just starting to grow in,`);
			} else if (slave.pubicHStyle === "in a strip") {
				r.push(`pubic hair, which is shaved into a strip,`);
			} else {
				r.push(`${slave.pubicHStyle} pubic hair`);
			}
			if (slave.pubicHStyle === "waxed") {
				r.push(`would be ${slave.pubicHColor} if present.`);
			} else {
				r.push(`is ${slave.pubicHColor}.`);
			}
		} else {
			r.push(`${His} groin is completely hairless.`);
		}
		option = options.addOption(r.join(" "), "pubicHColor", slave);
		if (hasPubes) {
			if (slave.pubicHColor !== slave.hColor) {
				option.addValue("Match the curtains", slave.hColor);
			}
			option.addValueList(makeAList(App.Medicine.Modification.Color.Primary.map(color => color.value)))
				.addCallbackToEach(billMod)
				.pulldown();

			// Style
			option = options.addOption(`Style ${his} pubic hair`, "pubicHStyle", slave);
			for (const fullness of App.Medicine.Modification.pubicStyles) {
				option.addValue(capFirstChar(fullness), fullness, billMod);
			}
			option.pulldown();
		}

		// Armpit hair
		r = [];
		const hasPitHair = (slave.underArmHStyle !== "bald" && slave.underArmHStyle !== "hairless" && slave.physicalAge >= _pubertyAge - 1);
		if (hasPitHair) {
			r.push(`${His}`);
			if (slave.physicalAge < _pubertyAge) {
				r.push(`wispy underarm hair`);
			} else {
				r.push(`${slave.underArmHStyle} underarm hair`);
			}
			if (slave.underArmHStyle === "waxed") {
				r.push(`would be ${slave.underArmHColor} if present.`);
			} else {
				r.push(`is ${slave.underArmHColor}.`);
			}
		} else {
			r.push(`${His} underarms are completely hairless.`);
		}
		option = options.addOption(r.join(" "), "underArmHColor", slave);
		if (hasPitHair) {
			if (slave.underArmHColor !== slave.hColor) {
				option.addValue("Match the hair", slave.hColor);
			}
			option.addValueList(makeAList(App.Medicine.Modification.Color.Primary.map(color => color.value)))
				.addCallbackToEach(billMod)
				.pulldown();

			// Style
			option = options.addOption(`Style ${his} armpit hair`, "underArmHStyle", slave);
			for (const fullness of App.Medicine.Modification.armpitStyles) {
				option.addValue(capFirstChar(fullness), fullness, billMod);
			}
			option.pulldown();
		}

		el.append(options.render());
		return el;
	}

	function billMod() {
		if (!cheat) {
			cashX(forceNeg(V.modCost), "slaveMod", slave);
		}
	}

	function makeAList(iterable) {
		return Array.from(iterable, (k => [capFirstChar(k), k]));
	}
};
