/**
 * UI for the Body Modification system/studio.  Refreshes without refreshing the passage.
 */
App.UI.toyShop = function() {
	const container = document.createElement("span");
	let buttPlugName;
	let buttPlugData;
	let selectedPlug;
	init();

	container.append(createPage());
	return container;

	function createPage() {
		const el = new DocumentFragment();
		el.append(intro());
		el.append(buttPlugs());
		return el;
	}

	function init() {
		buttPlugName = "";
		buttPlugData = {
			name: "",
			width: 1,
			length: 1
		};
	}

	function intro() {
		const el = new DocumentFragment();
		App.UI.DOM.appendNewElement("h1", el, "Toy Shop");
		App.UI.DOM.appendNewElement("div", el, `The room is filled with the smell of rubber, latex, and various synthetic materials and solvents.  A series of screens allows you to design toys of various shapes and sizes, and then produce them at scale.  A bin of defects sits in the corner, glistening a bit under a layer of lubrication.`, "scene-intro");
		return el;
	}


	function buttPlugs() {
		const el = new DocumentFragment();
		let linkArray;
		App.UI.DOM.appendNewElement("h2", el, "Buttplugs");
		const select = App.UI.DOM.appendNewElement("div", el, App.UI.DOM.link("Start a new design", () => {
			init();
			refresh();
		}));
		if (V.customItem.buttPlugs.size > 0) {
			select.append(selectDesign());
		}
		el.append(create());
		return el;

		function create() {
			const el = new DocumentFragment();
			const existingDesign = V.customItem.buttPlugs.get(buttPlugName);
			if (existingDesign) {
				el.append(descLocked());
			} else {
				el.append(desc());
			}
			if (buttPlugData.name) {
				el.append(title());
			}
			el.append(
				width(),
				length(),
			);

			if (existingDesign) {
				const build = App.UI.DOM.appendNewElement("div", el, `Send updated design to production and make sure all appropriate slaves are updated `);
				const linkArray = [];
				linkArray.push(
					App.UI.DOM.link(
						"Update the mold",
						() => { buildPlug(); }
					)
				);

				linkArray.push(
					App.UI.DOM.link(
						`Recall "${buttPlugName}"`,
						() => { deletePlug(); }
					)
				);
				build.append(App.UI.DOM.generateLinksStrip(linkArray));
			} else if (buttPlugName && buttPlugData.name){
				el.append(apply());
			}

			return el;

			function descLocked() {
				return App.UI.DOM.makeElement("div", `Description has already been selected for this model: "${buttPlugName}"`);
			}
		}
		function selectDesign() {
			const el = new DocumentFragment();
			const choice = App.UI.DOM.appendNewElement("span", el, ` or choose an existing design to edit `);
			const select = App.UI.DOM.appendNewElement("select", choice);
			let matchFound = false;
			for (const [key, values] of V.customItem.buttPlugs) {
				const option = App.UI.DOM.appendNewElement("option", select, values.name);
				option.value = key;
				if (option.value === buttPlugName) {
					option.selected = true;
					matchFound = true;
				}
			}
			if (!matchFound) {
				select.selectedIndex = -1;
			}
			select.onchange = () => {
				const O = select.options[select.selectedIndex];
				selectedPlug = O.value;
				buttPlugName = selectedPlug;
				buttPlugData = V.customItem.buttPlugs.get(selectedPlug);
				refresh();
			};
			return el;
		}

		function desc() {
			const value = App.UI.DOM.makeElement("div", `Enter shape here as it will appear in descriptions `);
			value.append(App.UI.DOM.makeTextBox(
				buttPlugName,
				v => {
					buttPlugName = v;
					buttPlugData.name = capFirstChar(v);
					refresh();
				}
			));
			App.UI.DOM.appendNewElement("span", value, ` Your slave has a standard ${buttPlugName ? buttPlugName : `spade-shaped plug`} wedged firmly in their asshole.`, "note");
			return value;
		}

		function title() {
			const title = App.UI.DOM.makeElement("div", `Enter title as it will appear in lists of choices `);
			title.append(App.UI.DOM.makeTextBox(
				buttPlugData.name,
				v => {
					buttPlugData.name = capFirstChar(v);
					refresh();
				}
			));
			return title;
		}

		function width() {
			const widthOptions = new Map([
				["standard", 1],
				["large", 2],
				["huge", 3],
			]);
			const width = App.UI.DOM.makeElement("div", `Select width `);
			linkArray = [];
			for (const [key, value] of widthOptions) {
				if (buttPlugData.width === value) {
					linkArray.push(
						App.UI.DOM.disabledLink(
							key,
							["Currently selected"]
						)
					);
				} else {
					linkArray.push(
						App.UI.DOM.link(
							key,
							() => {
								buttPlugData.width = value;
								refresh();
							}
						)
					);
				}
			}
			width.append(App.UI.DOM.generateLinksStrip(linkArray));
			return width;
		}

		function length() {
			const lengthOptions = new Map([
				["standard", 1],
				["long", 2],
			]);
			const length = App.UI.DOM.makeElement("div", `Select length `);
			linkArray = [];
			for (const [key, value] of lengthOptions) {
				if (buttPlugData.length === value) {
					linkArray.push(
						App.UI.DOM.disabledLink(
							key,
							["Currently selected"]
						)
					);
				} else {
					linkArray.push(
						App.UI.DOM.link(
							key,
							() => {
								buttPlugData.length = value;
								refresh();
							}
						)
					);
				}
			}
			length.append(App.UI.DOM.generateLinksStrip(linkArray));
			return length;
		}

		function buildPlug() {
			V.customItem.buttPlugs.set(buttPlugName, buttPlugData);
			init();
			refresh();
		}

		function apply() {
			const build = App.UI.DOM.appendNewElement("div", el, `Send design to production and make available for all slaves `);
			build.append(
				App.UI.DOM.link(
					"Start the mold",
					() => { buildPlug(); }
				)
			);
			return build;
		}

		function deletePlug() {
			V.customItem.buttPlugs.delete(buttPlugName);
			for (const slave of V.slaves) {
				if (slave.buttplug === buttPlugName) {
					slave.buttplug = "none";
				}
			}
			refresh();
		}
	}

	function refresh() {
		jQuery(container).empty().append(createPage());
	}
};
