globalThis.policies = (function() {
	return {
		countEugenicsSMRs: countEugenicsSMRs,
		cost: cost,
	};

	function countEugenicsSMRs() {
		let value = 0;
		if (V.policies.SMR.eugenics.faceSMR === 1) {
			value++;
		}
		if (V.policies.SMR.eugenics.heightSMR === 1) {
			value++;
		}
		if (V.policies.SMR.eugenics.intelligenceSMR === 1) {
			value++;
		}
		return value;
	}

	function cost() { return 5000; }
})();
/**
 * @param {string} category
 * @returns {JQuery}
 */
globalThis.policy = function(category) {
	const frag = new DocumentFragment;
	for (let policyVariable in App.Data.Policies.Selection[category]) {
		frag.append(policyElement(policyVariable));
	}
	return jQuery(`#${category}`).empty().append(frag);
	/**
	 * @param {string} policyVariable
	 * @returns {Node} el
	 */
	function policyElement(policyVariable) {
		let el = document.createElement("p");
		let div;
		let span;
		let link;
		const policyValue = _.get(V, policyVariable);
		/** @type {PolicySelector[]} */
		const policyObject = App.Data.Policies.Selection[category][policyVariable];
		if (policyValue === 0) {
			// apply
			for (let i = 0; i < policyObject.length; i++) {
				const p = policyObject[i];
				const enable = p.enable || 1;
				if (p.hasOwnProperty("requirements") && p.requirements === false) {
					continue;
				}
				div = document.createElement("div");
				span = document.createElement("span");

				// title
				span.style.fontWeight = "bold";
				if (p.hasOwnProperty("titleClass")) {
					span.classList.add(p.titleClass);
				}
				span.append(p.title);
				div.append(span);
				div.append(`: `);

				// Description text
				div.append(p.text);
				div.append(` `);

				// link
				if (!(p.hasOwnProperty("hide") && p.hide.button === 1)) {
					if (p.hasOwnProperty("requirements")) {
						if (p.requirements === true) {
							div.append(implement(p, enable));
						} else {
							link = App.UI.DOM.disabledLink("Implement", [`You do not meet the requirements, or passed a conflicting policy already`]);
							link.style.color = "white";
							div.append(link);
						}
					} else {
						div.append(implement(p, enable));
					}
				}
				el.append(div);
			}
		} else if (typeof policyValue === "string" || typeof policyValue === "number") {
			// repeal
			let i = 0;
			for (const pol in policyObject) {
				if (policyObject[pol].hasOwnProperty("enable") && policyObject[pol].enable === policyValue) {
					i = pol;
					break;
				}
			}
			const p = policyObject[i];
			if (p.hasOwnProperty("hide")) {
				if (p.hide.ifActivated === 1) {
					return el;
				}
			}

			let title;
			if (p.hasOwnProperty("activatedTitle")) {
				title = p.activatedTitle;
			} else {
				title = p.title;
			}
			let text;
			if (p.hasOwnProperty("activatedText")) {
				text = p.activatedText;
			} else {
				text = p.text;
			}
			div = document.createElement("div");
			span = document.createElement("span");

			// title
			span.style.fontWeight = "bold";
			span.append(title);
			div.append(span);
			div.append(`: `);

			// Description text
			div.append(text);
			div.append(` `);

			// link
			div.append(repeal(p));
			el.append(div);
		} else {
			throw `V.${policyVariable} not initialized properly, "${policyValue}"`;
		}

		return el;

		/**
		 * @param {PolicySelector} p The data object that describes the policy being considered.
		 * @returns {Node} Link to repeal.
		 */
		function repeal(p) {
			const frag = new DocumentFragment;
			let check = canAfford();
			if (!(p.hasOwnProperty("hide") && p.hide.button === 1)) {
				if (check === true) {
					link = App.UI.DOM.link(
						"Repeal",
						() => {
							if (V.rep >= 1000) {
								_.set(V, policyVariable, 0);
								applyCost();
								if (p.hasOwnProperty("onRepeal")) {
									p.onRepeal();
								}
							}
							policy(category);
						}
					);
					link.style.color = "yellow";
				} else {
					link = App.UI.DOM.disabledLink("Repeal", [`You do not have enough ${check}`]);
					link.style.color = "red";
				}
				frag.append(link);
			}

			if (p.hasOwnProperty("activatedNote")) {
				frag.append(App.UI.DOM.makeElement("div", p.activatedNote, ["note", "indent"]));
			}
			return frag;
		}
		/**
		 * @param {PolicySelector} p The data object that describes the policy being considered.
		 * @param {number|string} enable value to set the policy to in order to switch it on.
		 * @returns {Node} Link to implement.
		 */
		function implement(p, enable) {
			let check = canAfford();
			const frag = new DocumentFragment;
			if (check === true) {
				link = App.UI.DOM.link(
					"Implement",
					() => {
						if (V.rep >= 1000) {
							_.set(V, policyVariable, enable);
							applyCost();
							if (p.hasOwnProperty("onImplementation")) {
								p.onImplementation();
							}
						}
						policy(category);
					}
				);
				link.style.color = "green";
			} else {
				link = App.UI.DOM.disabledLink("Implement", [`You do not have enough ${check}`]);
				link.style.color = "red";
			}
			frag.append(link);
			if (p.hasOwnProperty("note")) {
				frag.append(App.UI.DOM.makeElement("div", p.note, ["note", "indent"]));
			}
			return frag;
		}

		function canAfford() {
			if (V.cash < 5000) {
				return "cash";
			} else if (V.rep < 1000 && !["EducationPolicies"].includes(category)) {
				return "reputation";
			}
			return true;
		}

		function applyCost() {
			cashX(-5000, "policies");
			if (!["EducationPolicies"].includes(category)) {
				repX(-1000, "policies");
			}
		}
	}
};
